# pylint: disable=too-many-locals

from risc_generator import Protections, FPQ1Contract, Premium
from risc_generator.content import en
from tests.domain.contract.contract_base import ContractTestBase

INCLUDE = "include"
EXCLUDE = "exclude"


class ContractDeclarationTest(ContractTestBase):
    def test_givenAENContractLiabilityOnly_whenGenerateItem4Text_thenReturnProperENText(self):
        a_liability_coverage_premium = 1
        protection_premiums = {"LiabilityCoveragePremium": a_liability_coverage_premium}
        a_premium = Premium(**protection_premiums)

        a_dict_of_protections = {
            "LiabilityCoverage": 1000000,
            "PropertyDamageB1Coverage": EXCLUDE,
            "PropertyDamageB2Coverage": EXCLUDE,
            "PropertyDamageB3Coverage": EXCLUDE,
            "PropertyDamageB4Coverage": EXCLUDE,
        }
        a_protections = Protections(protections=a_dict_of_protections)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=a_premium,
            financing=self.a_creditor,
            locale=self.a_en_locale,
        )

        actual = en.declaration_articles.generate_item_4_text(
            protections=fpq1_contract.protections, premium=fpq1_contract.premium
        )

        a_insurer_name = "Assureur"
        fidelity_program_name = "FIDELITY"
        a_new_page = "<###NEW_PAGE###>"

        expected = (
            "Item 4. COVERAGE / PERILS: The perils covered by the insurance contract are those for which an amount "
            "of insurance, a deductible or an insurance premium is shown in the table below. Coverage is subject to "
            "the conditions set out in the insurance contract.\n"
            "PREMIUM\n"
            "\t Section A - CIVIL LIABILITY -\n"
            f"Property damage or bodily injury to another person - Amount of insurance: 1000000$ "
            f"{a_liability_coverage_premium}$\n"
            "\t Section B - DAMAGE TO INSURED VEHICLES\n"
            "Protection 1: All risks Excluded\n"
            "Protection 2: Risk of collision and rollover Excluded\n"
            "Protection 3: All risks except collision or rollover Excluded\n"
            "Protection 4: Specific risks Excluded\n"
            f"{a_new_page}\n"
            f"\t {fidelity_program_name} program\n"
            f"Q.E.F. N 41\n"
            f"- Change to deductibles (Section B): Included\n"
            f"This endorsement makes the following changes to the deductible amounts under Section B\n"
            f"\t - No deductible to pay in the case of a hit-and-run offence if a police report was issued.\n"
            f"\t - No deductible to pay in the case of a total loss.\n"
            f"\t - No deductible to pay in the case of a windshield repair except in the case of a windshield "
            f"replacement.\n "
            f"\t - Should you incur a loss that affects two or more insurance products insured at "
            f"{a_insurer_name}, you will have but one deductible to pay, the highest applicable.\n"
            "Premium due date: according to the agreed terms of payment\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAENContractLiabilityWithB1_whenGenerateItem4Text_thenReturnProperENText(self):
        a_liability_coverage_premium = 1
        a_b1_premium = 2
        protection_premiums = {
            "LiabilityCoveragePremium": a_liability_coverage_premium,
            "PropertyDamageB1CoveragePremium": a_b1_premium,
        }
        a_premium = Premium(**protection_premiums)

        a_dict_of_protections = {
            "LiabilityCoverage": 1000000,
            "PropertyDamageB1Coverage": 250,
            "PropertyDamageB2Coverage": EXCLUDE,
            "PropertyDamageB3Coverage": EXCLUDE,
            "PropertyDamageB4Coverage": EXCLUDE,
        }
        a_protections = Protections(protections=a_dict_of_protections)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=a_premium,
            financing=self.a_creditor,
            locale=self.a_en_locale,
        )

        actual = en.declaration_articles.generate_item_4_text(
            protections=fpq1_contract.protections, premium=fpq1_contract.premium
        )

        a_insurer_name = "Assureur"
        fidelity_program_name = "FIDELITY"
        a_new_page = "<###NEW_PAGE###>"

        expected = (
            "Item 4. COVERAGE / PERILS: The perils covered by the insurance contract are those for which an amount "
            "of insurance, a deductible or an insurance premium is shown in the table below. Coverage is subject to "
            "the conditions set out in the insurance contract.\n"
            "PREMIUM\n"
            "\t Section A - CIVIL LIABILITY -\n"
            f"Property damage or bodily injury to another person - Amount of insurance: 1000000$ "
            f"{a_liability_coverage_premium}$\n"
            "\t Section B - DAMAGE TO INSURED VEHICLES\n"
            "Protection 1: All risks - Deductible 250$ per claim 2$\n"
            "Protection 2: Risk of collision and rollover Excluded\n"
            "Protection 3: All risks except collision or rollover Excluded\n"
            "Protection 4: Specific risks Excluded\n"
            f"{a_new_page}\n"
            f"\t {fidelity_program_name} program\n"
            f"Q.E.F. N 41\n"
            f"- Change to deductibles (Section B): Included\n"
            f"This endorsement makes the following changes to the deductible amounts under Section B\n"
            f"\t - No deductible to pay in the case of a hit-and-run offence if a police report was issued.\n"
            f"\t - No deductible to pay in the case of a total loss.\n"
            f"\t - No deductible to pay in the case of a windshield repair except in the case of a windshield "
            f"replacement.\n "
            f"\t - Should you incur a loss that affects two or more insurance products insured at "
            f"{a_insurer_name}, you will have but one deductible to pay, the highest applicable.\n"
            "Premium due date: according to the agreed terms of payment\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAENContractLiabilityWithB2B3_whenGenerateItem4Text_thenReturnProperENText(self):
        a_liability_coverage_premium = 1
        a_b2_premium = 2
        a_b3_premium = 3
        protection_premiums = {
            "LiabilityCoveragePremium": a_liability_coverage_premium,
            "PropertyDamageB2CoveragePremium": a_b2_premium,
            "PropertyDamageB3CoveragePremium": a_b3_premium,
        }
        a_premium = Premium(**protection_premiums)

        a_dict_of_protections = {
            "LiabilityCoverage": 1000000,
            "PropertyDamageB1Coverage": INCLUDE,
            "PropertyDamageB2Coverage": 250,
            "PropertyDamageB3Coverage": 250,
            "PropertyDamageB4Coverage": EXCLUDE,
        }
        a_protections = Protections(protections=a_dict_of_protections)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=a_premium,
            financing=self.a_creditor,
            locale=self.a_en_locale,
        )

        actual = en.declaration_articles.generate_item_4_text(
            protections=fpq1_contract.protections, premium=fpq1_contract.premium
        )

        a_insurer_name = "Assureur"
        fidelity_program_name = "FIDELITY"
        a_new_page = "<###NEW_PAGE###>"

        expected = (
            "Item 4. COVERAGE / PERILS: The perils covered by the insurance contract are those for which an amount "
            "of insurance, a deductible or an insurance premium is shown in the table below. Coverage is subject to "
            "the conditions set out in the insurance contract.\n"
            "PREMIUM\n"
            "\t Section A - CIVIL LIABILITY -\n"
            f"Property damage or bodily injury to another person - Amount of insurance: 1000000$ "
            f"{a_liability_coverage_premium}$\n"
            "\t Section B - DAMAGE TO INSURED VEHICLES\n"
            "Protection 1: All risks Included\n"
            "Protection 2: Risk of collision and rollover - Deductible 250$ per claim 2$\n"
            "Protection 3: All risks except collision or rollover - Deductible 250$ per claim 3$\n"
            "Protection 4: Specific risks Excluded\n"
            f"{a_new_page}\n"
            f"\t {fidelity_program_name} program\n"
            f"Q.E.F. N 41\n"
            f"- Change to deductibles (Section B): Included\n"
            f"This endorsement makes the following changes to the deductible amounts under Section B\n"
            f"\t - No deductible to pay in the case of a hit-and-run offence if a police report was issued.\n"
            f"\t - No deductible to pay in the case of a total loss.\n"
            f"\t - No deductible to pay in the case of a windshield repair except in the case of a windshield "
            f"replacement.\n "
            f"\t - Should you incur a loss that affects two or more insurance products insured at "
            f"{a_insurer_name}, you will have but one deductible to pay, the highest applicable.\n"
            "Premium due date: according to the agreed terms of payment\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAENContractLiabilityWithB2B4_whenGenerateItem4Text_thenReturnProperENText(self):
        a_liability_coverage_premium = 1
        a_b2_premium = 2
        a_b4_premium = 3
        protection_premiums = {
            "LiabilityCoveragePremium": a_liability_coverage_premium,
            "PropertyDamageB2CoveragePremium": a_b2_premium,
            "PropertyDamageB4CoveragePremium": a_b4_premium,
        }
        a_premium = Premium(**protection_premiums)

        a_dict_of_protections = {
            "LiabilityCoverage": 1000000,
            "PropertyDamageB1Coverage": INCLUDE,
            "PropertyDamageB2Coverage": 250,
            "PropertyDamageB3Coverage": EXCLUDE,
            "PropertyDamageB4Coverage": 250,
        }
        a_protections = Protections(protections=a_dict_of_protections)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=a_premium,
            financing=self.a_creditor,
            locale=self.a_en_locale,
        )

        actual = en.declaration_articles.generate_item_4_text(
            protections=fpq1_contract.protections, premium=fpq1_contract.premium
        )

        a_insurer_name = "Assureur"
        fidelity_program_name = "FIDELITY"
        a_new_page = "<###NEW_PAGE###>"

        expected = (
            "Item 4. COVERAGE / PERILS: The perils covered by the insurance contract are those for which an amount "
            "of insurance, a deductible or an insurance premium is shown in the table below. Coverage is subject to "
            "the conditions set out in the insurance contract.\n"
            "PREMIUM\n"
            "\t Section A - CIVIL LIABILITY -\n"
            f"Property damage or bodily injury to another person - Amount of insurance: 1000000$ "
            f"{a_liability_coverage_premium}$\n"
            "\t Section B - DAMAGE TO INSURED VEHICLES\n"
            "Protection 1: All risks Included\n"
            "Protection 2: Risk of collision and rollover - Deductible 250$ per claim 2$\n"
            "Protection 3: All risks except collision or rollover Excluded\n"
            "Protection 4: Specific risks - Deductible 250$ per claim 3$\n"
            f"{a_new_page}\n"
            f"\t {fidelity_program_name} program\n"
            f"Q.E.F. N 41\n"
            f"- Change to deductibles (Section B): Included\n"
            f"This endorsement makes the following changes to the deductible amounts under Section B\n"
            f"\t - No deductible to pay in the case of a hit-and-run offence if a police report was issued.\n"
            f"\t - No deductible to pay in the case of a total loss.\n"
            f"\t - No deductible to pay in the case of a windshield repair except in the case of a windshield "
            f"replacement.\n "
            f"\t - Should you incur a loss that affects two or more insurance products insured at "
            f"{a_insurer_name}, you will have but one deductible to pay, the highest applicable.\n"
            "Premium due date: according to the agreed terms of payment\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAENContractWithEndorsementFAQ33_whenGenerateItem4Text_thenReturnProperENText(self):
        a_liability_coverage_premium = 1
        a_b2_premium = 2
        a_b3_premium = 3
        a_faq_premium = 4
        protection_premiums = {
            "LiabilityCoveragePremium": a_liability_coverage_premium,
            "PropertyDamageB2CoveragePremium": a_b2_premium,
            "PropertyDamageB3CoveragePremium": a_b3_premium,
            "FAQ33CoveragePremium": a_faq_premium,
        }
        a_premium = Premium(**protection_premiums)

        a_dict_of_protections = {
            "LiabilityCoverage": 1000000,
            "PropertyDamageB1Coverage": INCLUDE,
            "PropertyDamageB2Coverage": 250,
            "PropertyDamageB3Coverage": 250,
            "PropertyDamageB4Coverage": EXCLUDE,
            "FAQ33Coverage": INCLUDE,
        }
        a_protections = Protections(protections=a_dict_of_protections)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=a_premium,
            financing=self.a_creditor,
            locale=self.a_en_locale,
        )

        actual = en.declaration_articles.generate_item_4_text(
            protections=fpq1_contract.protections, premium=fpq1_contract.premium
        )

        a_insurer_name = "Assureur"
        fidelity_program_name = "FIDELITY"
        a_new_page = "<###NEW_PAGE###>"
        a_roadside_program_name = "ABC"

        expected = (
            "Item 4. COVERAGE / PERILS: The perils covered by the insurance contract are those for which an amount "
            "of insurance, a deductible or an insurance premium is shown in the table below. Coverage is subject to "
            "the conditions set out in the insurance contract.\n"
            "PREMIUM\n"
            "\t Section A - CIVIL LIABILITY -\n"
            f"Property damage or bodily injury to another person - Amount of insurance: 1000000$ "
            f"{a_liability_coverage_premium}$\n"
            "\t Section B - DAMAGE TO INSURED VEHICLES\n"
            "Protection 1: All risks Included\n"
            "Protection 2: Risk of collision and rollover - Deductible 250$ per claim 2$\n"
            "Protection 3: All risks except collision or rollover - Deductible 250$ per claim 3$\n"
            "Protection 4: Specific risks Excluded\n"
            f"{a_new_page}\n"
            "Endorsement(s): Q.E.F. N 33 - Insurance for roadside assistance expenses\n"
            f"{a_roadside_program_name} Roadside Assistance Program, available 24 hours a day, 7 days a week\n"
            "Maximum radius 50 km -\n"
            f"In the event of a breakdown, the towing distance is increased to 100 km in the Laurentian Park, "
            "150 km in La Vérendrye Park and 100 km in Gaspésie National Park\n"
            f"Maximum number of events per insurance year 4 4$\n"
            "Limit per breakdown service 60$(1) - For a towing or a stuck vehicle, this limit is increased to 100$(1)\n"
            "Limit per policy year 400$(1)\n"
            "(1) These limits apply only when, on an exceptional basis, towing or breakdown service is not available "
            "through the Roadside Assistance Program\n"
            "See your brochure for all additional services available to you\n"
            f"\t {fidelity_program_name} program\n"
            f"Q.E.F. N 41\n"
            f"- Change to deductibles (Section B): Included\n"
            f"This endorsement makes the following changes to the deductible amounts under Section B\n"
            f"\t - No deductible to pay in the case of a hit-and-run offence if a police report was issued.\n"
            f"\t - No deductible to pay in the case of a total loss.\n"
            f"\t - No deductible to pay in the case of a windshield repair except in the case of a windshield "
            f"replacement.\n "
            f"\t - Should you incur a loss that affects two or more insurance products insured at "
            f"{a_insurer_name}, you will have but one deductible to pay, the highest applicable.\n"
            "Premium due date: according to the agreed terms of payment\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAENContractWithEndorsementFAQ20A_whenGenerateItem4Text_thenReturnProperENText(self):
        a_liability_coverage_premium = 1
        a_b2_premium = 2
        a_b3_premium = 3
        a_faq_premium = 4
        protection_premiums = {
            "LiabilityCoveragePremium": a_liability_coverage_premium,
            "PropertyDamageB2CoveragePremium": a_b2_premium,
            "PropertyDamageB3CoveragePremium": a_b3_premium,
            "FAQ20ACoveragePremium": a_faq_premium,
        }
        a_premium = Premium(**protection_premiums)

        a_dict_of_protections = {
            "LiabilityCoverage": 1000000,
            "PropertyDamageB1Coverage": INCLUDE,
            "PropertyDamageB2Coverage": 250,
            "PropertyDamageB3Coverage": 250,
            "PropertyDamageB4Coverage": EXCLUDE,
            "FAQ20ACoverage": INCLUDE,
        }
        a_protections = Protections(protections=a_dict_of_protections)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=a_premium,
            financing=self.a_creditor,
            locale=self.a_en_locale,
        )

        actual = en.declaration_articles.generate_item_4_text(
            protections=fpq1_contract.protections, premium=fpq1_contract.premium
        )

        a_insurer_name = "Assureur"
        fidelity_program_name = "FIDELITY"
        a_new_page = "<###NEW_PAGE###>"

        expected = (
            "Item 4. COVERAGE / PERILS: The perils covered by the insurance contract are those for which an amount "
            "of insurance, a deductible or an insurance premium is shown in the table below. Coverage is subject to "
            "the conditions set out in the insurance contract.\n"
            "PREMIUM\n"
            "\t Section A - CIVIL LIABILITY -\n"
            f"Property damage or bodily injury to another person - Amount of insurance: 1000000$ "
            f"{a_liability_coverage_premium}$\n"
            "\t Section B - DAMAGE TO INSURED VEHICLES\n"
            "Protection 1: All risks Included\n"
            "Protection 2: Risk of collision and rollover - Deductible 250$ per claim 2$\n"
            "Protection 3: All risks except collision or rollover - Deductible 250$ per claim 3$\n"
            "Protection 4: Specific risks Excluded\n"
            f"{a_new_page}\n"
            "Endorsement(s): Q.E.F. N 20a - Travel Expense (Extended Form) (Chapter B) 4$\n"
            "Travel expenses 50$ per day, total limit of 1500$ per claim\n"
            "Other covered expenses during a trip - 750$\n"
            f"\t {fidelity_program_name} program\n"
            f"Q.E.F. N 41\n"
            f"- Change to deductibles (Section B): Included\n"
            f"This endorsement makes the following changes to the deductible amounts under Section B\n"
            f"\t - No deductible to pay in the case of a hit-and-run offence if a police report was issued.\n"
            f"\t - No deductible to pay in the case of a total loss.\n"
            f"\t - No deductible to pay in the case of a windshield repair except in the case of a windshield "
            f"replacement.\n "
            f"\t - Should you incur a loss that affects two or more insurance products insured at "
            f"{a_insurer_name}, you will have but one deductible to pay, the highest applicable.\n"
            "Premium due date: according to the agreed terms of payment\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAENContractWithEndorsementFAQ20_whenGenerateItem4Text_thenReturnProperENText(self):
        a_liability_coverage_premium = 1
        a_b2_premium = 2
        a_b3_premium = 3
        a_faq_premium = 4
        protection_premiums = {
            "LiabilityCoveragePremium": a_liability_coverage_premium,
            "PropertyDamageB2CoveragePremium": a_b2_premium,
            "PropertyDamageB3CoveragePremium": a_b3_premium,
            "FAQ20CoveragePremium": a_faq_premium,
        }
        a_premium = Premium(**protection_premiums)

        a_dict_of_protections = {
            "LiabilityCoverage": 1000000,
            "PropertyDamageB1Coverage": INCLUDE,
            "PropertyDamageB2Coverage": 250,
            "PropertyDamageB3Coverage": 250,
            "PropertyDamageB4Coverage": EXCLUDE,
            "FAQ20Coverage": INCLUDE,
        }
        a_protections = Protections(protections=a_dict_of_protections)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=a_premium,
            financing=self.a_creditor,
            locale=self.a_en_locale,
        )

        actual = en.declaration_articles.generate_item_4_text(
            protections=fpq1_contract.protections, premium=fpq1_contract.premium
        )

        a_insurer_name = "Assureur"
        fidelity_program_name = "FIDELITY"
        a_new_page = "<###NEW_PAGE###>"

        expected = (
            "Item 4. COVERAGE / PERILS: The perils covered by the insurance contract are those for which an amount "
            "of insurance, a deductible or an insurance premium is shown in the table below. Coverage is subject to "
            "the conditions set out in the insurance contract.\n"
            "PREMIUM\n"
            "\t Section A - CIVIL LIABILITY -\n"
            f"Property damage or bodily injury to another person - Amount of insurance: 1000000$ "
            f"{a_liability_coverage_premium}$\n"
            "\t Section B - DAMAGE TO INSURED VEHICLES\n"
            "Protection 1: All risks Included\n"
            "Protection 2: Risk of collision and rollover - Deductible 250$ per claim 2$\n"
            "Protection 3: All risks except collision or rollover - Deductible 250$ per claim 3$\n"
            "Protection 4: Specific risks Excluded\n"
            f"{a_new_page}\n"
            "Endorsement(s): Q.E.F. N 20 - Travel Expense (Extended Form) (Chapter B) 4$\n"
            "Travel expenses 50$ per day, total limit of 1500$ per claim\n"
            "Other covered expenses during a trip - 750$\n"
            f"\t {fidelity_program_name} program\n"
            f"Q.E.F. N 41\n"
            f"- Change to deductibles (Section B): Included\n"
            f"This endorsement makes the following changes to the deductible amounts under Section B\n"
            f"\t - No deductible to pay in the case of a hit-and-run offence if a police report was issued.\n"
            f"\t - No deductible to pay in the case of a total loss.\n"
            f"\t - No deductible to pay in the case of a windshield repair except in the case of a windshield "
            f"replacement.\n "
            f"\t - Should you incur a loss that affects two or more insurance products insured at "
            f"{a_insurer_name}, you will have but one deductible to pay, the highest applicable.\n"
            "Premium due date: according to the agreed terms of payment\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAENContractWithEndorsementFAQ34_whenGenerateItem4Text_thenReturnProperENText(self):
        a_liability_coverage_premium = 1
        a_b2_premium = 2
        a_b3_premium = 3
        a_faq_premium = 4
        protection_premiums = {
            "LiabilityCoveragePremium": a_liability_coverage_premium,
            "PropertyDamageB2CoveragePremium": a_b2_premium,
            "PropertyDamageB3CoveragePremium": a_b3_premium,
            "FAQ34CoveragePremium": a_faq_premium,
        }
        a_premium = Premium(**protection_premiums)

        a_dict_of_protections = {
            "LiabilityCoverage": 1000000,
            "PropertyDamageB1Coverage": INCLUDE,
            "PropertyDamageB2Coverage": 250,
            "PropertyDamageB3Coverage": 250,
            "PropertyDamageB4Coverage": EXCLUDE,
            "FAQ34Coverage": INCLUDE,
        }
        a_protections = Protections(protections=a_dict_of_protections)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=a_premium,
            financing=self.a_creditor,
            locale=self.a_en_locale,
        )

        actual = en.declaration_articles.generate_item_4_text(
            protections=fpq1_contract.protections, premium=fpq1_contract.premium
        )

        a_insurer_name = "Assureur"
        fidelity_program_name = "FIDELITY"
        a_new_page = "<###NEW_PAGE###>"

        expected = (
            "Item 4. COVERAGE / PERILS: The perils covered by the insurance contract are those for which an amount "
            "of insurance, a deductible or an insurance premium is shown in the table below. Coverage is subject to "
            "the conditions set out in the insurance contract.\n"
            "PREMIUM\n"
            "\t Section A - CIVIL LIABILITY -\n"
            f"Property damage or bodily injury to another person - Amount of insurance: 1000000$ "
            f"{a_liability_coverage_premium}$\n"
            "\t Section B - DAMAGE TO INSURED VEHICLES\n"
            "Protection 1: All risks Included\n"
            "Protection 2: Risk of collision and rollover - Deductible 250$ per claim 2$\n"
            "Protection 3: All risks except collision or rollover - Deductible 250$ per claim 3$\n"
            "Protection 4: Specific risks Excluded\n"
            f"{a_new_page}\n"
            "Endorsement(s): Q.E.F. N 34 - Accident benefits insurance: 4$\n"
            "Division 1: Subdivision A and B - Death and dismemberment benefits: 20000$ principal sum\n"
            "Subdivision C - Reimbursement of medical expenses: 2000$ per person\n"
            "Division 2: Total disability benefits: Not covered\n"
            f"\t {fidelity_program_name} program\n"
            f"Q.E.F. N 41\n"
            f"- Change to deductibles (Section B): Included\n"
            f"This endorsement makes the following changes to the deductible amounts under Section B\n"
            f"\t - No deductible to pay in the case of a hit-and-run offence if a police report was issued.\n"
            f"\t - No deductible to pay in the case of a total loss.\n"
            f"\t - No deductible to pay in the case of a windshield repair except in the case of a windshield "
            f"replacement.\n "
            f"\t - Should you incur a loss that affects two or more insurance products insured at "
            f"{a_insurer_name}, you will have but one deductible to pay, the highest applicable.\n"
            "Premium due date: according to the agreed terms of payment\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAENContractWithEndorsementFAQ34AB_whenGenerateItem4Text_thenReturnProperENText(self):
        a_liability_coverage_premium = 1
        a_b2_premium = 2
        a_b3_premium = 3
        a_faq_premium = 4
        protection_premiums = {
            "LiabilityCoveragePremium": a_liability_coverage_premium,
            "PropertyDamageB2CoveragePremium": a_b2_premium,
            "PropertyDamageB3CoveragePremium": a_b3_premium,
            "FAQ34ABCoveragePremium": a_faq_premium,
        }
        a_premium = Premium(**protection_premiums)

        a_dict_of_protections = {
            "LiabilityCoverage": 1000000,
            "PropertyDamageB1Coverage": INCLUDE,
            "PropertyDamageB2Coverage": 250,
            "PropertyDamageB3Coverage": 250,
            "PropertyDamageB4Coverage": EXCLUDE,
            "FAQ34ABCoverage": INCLUDE,
        }
        a_protections = Protections(protections=a_dict_of_protections)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=a_premium,
            financing=self.a_creditor,
            locale=self.a_en_locale,
        )

        actual = en.declaration_articles.generate_item_4_text(
            protections=fpq1_contract.protections, premium=fpq1_contract.premium
        )

        a_insurer_name = "Assureur"
        fidelity_program_name = "FIDELITY"
        a_new_page = "<###NEW_PAGE###>"

        expected = (
            "Item 4. COVERAGE / PERILS: The perils covered by the insurance contract are those for which an amount "
            "of insurance, a deductible or an insurance premium is shown in the table below. Coverage is subject to "
            "the conditions set out in the insurance contract.\n"
            "PREMIUM\n"
            "\t Section A - CIVIL LIABILITY -\n"
            f"Property damage or bodily injury to another person - Amount of insurance: 1000000$ "
            f"{a_liability_coverage_premium}$\n"
            "\t Section B - DAMAGE TO INSURED VEHICLES\n"
            "Protection 1: All risks Included\n"
            "Protection 2: Risk of collision and rollover - Deductible 250$ per claim 2$\n"
            "Protection 3: All risks except collision or rollover - Deductible 250$ per claim 3$\n"
            "Protection 4: Specific risks Excluded\n"
            f"{a_new_page}\n"
            "Endorsement(s): Q.E.F. N 34ab - Accident benefits insurance: 4$\n"
            "Division 1: Subdivision A and B - Death and dismemberment benefits: 20000$ principal sum\n"
            "Subdivision C - Reimbursement of medical expenses: 2000$ per person\n"
            "Division 2: Total disability benefits: Not covered\n"
            f"\t {fidelity_program_name} program\n"
            f"Q.E.F. N 41\n"
            f"- Change to deductibles (Section B): Included\n"
            f"This endorsement makes the following changes to the deductible amounts under Section B\n"
            f"\t - No deductible to pay in the case of a hit-and-run offence if a police report was issued.\n"
            f"\t - No deductible to pay in the case of a total loss.\n"
            f"\t - No deductible to pay in the case of a windshield repair except in the case of a windshield "
            f"replacement.\n "
            f"\t - Should you incur a loss that affects two or more insurance products insured at "
            f"{a_insurer_name}, you will have but one deductible to pay, the highest applicable.\n"
            "Premium due date: according to the agreed terms of payment\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAENContractWithEndorsementFAQ43_whenGenerateItem4Text_thenReturnProperENText(self):
        a_liability_coverage_premium = 1
        a_b2_premium = 2
        a_b3_premium = 3
        a_faq_premium = 4
        protection_premiums = {
            "LiabilityCoveragePremium": a_liability_coverage_premium,
            "PropertyDamageB2CoveragePremium": a_b2_premium,
            "PropertyDamageB3CoveragePremium": a_b3_premium,
            "FAQ43CoveragePremium": a_faq_premium,
        }
        a_premium = Premium(**protection_premiums)

        a_dict_of_protections = {
            "LiabilityCoverage": 1000000,
            "PropertyDamageB1Coverage": INCLUDE,
            "PropertyDamageB2Coverage": 250,
            "PropertyDamageB3Coverage": 250,
            "PropertyDamageB4Coverage": EXCLUDE,
            "FAQ43Coverage": INCLUDE,
        }
        a_protections = Protections(protections=a_dict_of_protections)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=a_premium,
            financing=self.a_creditor,
            locale=self.a_en_locale,
        )

        actual = en.declaration_articles.generate_item_4_text(
            protections=fpq1_contract.protections, premium=fpq1_contract.premium
        )

        a_insurer_name = "Assureur"
        fidelity_program_name = "FIDELITY"
        a_new_page = "<###NEW_PAGE###>"

        expected = (
            "Item 4. COVERAGE / PERILS: The perils covered by the insurance contract are those for which an amount "
            "of insurance, a deductible or an insurance premium is shown in the table below. Coverage is subject to "
            "the conditions set out in the insurance contract.\n"
            "PREMIUM\n"
            "\t Section A - CIVIL LIABILITY -\n"
            f"Property damage or bodily injury to another person - Amount of insurance: 1000000$ "
            f"{a_liability_coverage_premium}$\n"
            "\t Section B - DAMAGE TO INSURED VEHICLES\n"
            "Protection 1: All risks Included\n"
            "Protection 2: Risk of collision and rollover - Deductible 250$ per claim 2$\n"
            "Protection 3: All risks except collision or rollover - Deductible 250$ per claim 3$\n"
            "Protection 4: Specific risks Excluded\n"
            f"{a_new_page}\n"
            "Endorsement(s): Q.E.F. N 43a - Change to indemnity (Section B) - Partial loss - New parts 1$\n"
            "Q.E.F. N 43e - Change to indemnity (Section B) - Total loss - Replacement cost 3$\n"
            f"\t {fidelity_program_name} program\n"
            f"Q.E.F. N 41\n"
            f"- Change to deductibles (Section B): Included\n"
            f"This endorsement makes the following changes to the deductible amounts under Section B\n"
            f"\t - No deductible to pay in the case of a hit-and-run offence if a police report was issued.\n"
            f"\t - No deductible to pay in the case of a total loss.\n"
            f"\t - No deductible to pay in the case of a windshield repair except in the case of a windshield "
            f"replacement.\n "
            f"\t - Should you incur a loss that affects two or more insurance products insured at "
            f"{a_insurer_name}, you will have but one deductible to pay, the highest applicable.\n"
            "Premium due date: according to the agreed terms of payment\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAENContractWithEndorsementFAQ2_whenGenerateItem4Text_thenReturnProperENText(self):
        a_liability_coverage_premium = 1
        a_b2_premium = 2
        a_b3_premium = 3
        a_faq_premium = 4
        protection_premiums = {
            "LiabilityCoveragePremium": a_liability_coverage_premium,
            "PropertyDamageB2CoveragePremium": a_b2_premium,
            "PropertyDamageB3CoveragePremium": a_b3_premium,
            "FAQ2CoveragePremium": a_faq_premium,
        }
        a_premium = Premium(**protection_premiums)

        a_dict_of_protections = {
            "LiabilityCoverage": 1000000,
            "PropertyDamageB1Coverage": INCLUDE,
            "PropertyDamageB2Coverage": 250,
            "PropertyDamageB3Coverage": 250,
            "PropertyDamageB4Coverage": EXCLUDE,
            "FAQ2Coverage": INCLUDE,
        }
        a_protections = Protections(protections=a_dict_of_protections)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=a_premium,
            financing=self.a_creditor,
            locale=self.a_en_locale,
        )

        actual = en.declaration_articles.generate_item_4_text(
            protections=fpq1_contract.protections, premium=fpq1_contract.premium
        )

        a_insurer_name = "Assureur"
        fidelity_program_name = "FIDELITY"
        a_new_page = "<###NEW_PAGE###>"

        expected = (
            "Item 4. COVERAGE / PERILS: The perils covered by the insurance contract are those for which an amount "
            "of insurance, a deductible or an insurance premium is shown in the table below. Coverage is subject to "
            "the conditions set out in the insurance contract.\n"
            "PREMIUM\n"
            "\t Section A - CIVIL LIABILITY -\n"
            f"Property damage or bodily injury to another person - Amount of insurance: 1000000$ "
            f"{a_liability_coverage_premium}$\n"
            "\t Section B - DAMAGE TO INSURED VEHICLES\n"
            "Protection 1: All risks Included\n"
            "Protection 2: Risk of collision and rollover - Deductible 250$ per claim 2$\n"
            "Protection 3: All risks except collision or rollover - Deductible 250$ per claim 3$\n"
            "Protection 4: Specific risks Excluded\n"
            f"{a_new_page}\n"
            "Endorsement(s): Q.E.F. N 2 - Vehicles of which named insured is not owner and when driven by named "
            "drivers (Section A) Included\n"
            f"\t {fidelity_program_name} program\n"
            f"Q.E.F. N 41\n"
            f"- Change to deductibles (Section B): Included\n"
            f"This endorsement makes the following changes to the deductible amounts under Section B\n"
            f"\t - No deductible to pay in the case of a hit-and-run offence if a police report was issued.\n"
            f"\t - No deductible to pay in the case of a total loss.\n"
            f"\t - No deductible to pay in the case of a windshield repair except in the case of a windshield "
            f"replacement.\n "
            f"\t - Should you incur a loss that affects two or more insurance products insured at "
            f"{a_insurer_name}, you will have but one deductible to pay, the highest applicable.\n"
            "Premium due date: according to the agreed terms of payment\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAENContractWithEndorsementFAQ5A_whenGenerateItem4Text_thenReturnProperENText(self):
        a_liability_coverage_premium = 1
        a_b2_premium = 2
        a_b3_premium = 3
        a_faq_premium = 4
        protection_premiums = {
            "LiabilityCoveragePremium": a_liability_coverage_premium,
            "PropertyDamageB2CoveragePremium": a_b2_premium,
            "PropertyDamageB3CoveragePremium": a_b3_premium,
            "FAQ5ACoveragePremium": a_faq_premium,
        }
        a_premium = Premium(**protection_premiums)

        a_dict_of_protections = {
            "LiabilityCoverage": 1000000,
            "PropertyDamageB1Coverage": INCLUDE,
            "PropertyDamageB2Coverage": 250,
            "PropertyDamageB3Coverage": 250,
            "PropertyDamageB4Coverage": EXCLUDE,
            "FAQ5ACoverage": INCLUDE,
        }
        a_protections = Protections(protections=a_dict_of_protections)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=a_premium,
            financing=self.a_creditor,
            locale=self.a_en_locale,
        )

        actual = en.declaration_articles.generate_item_4_text(
            protections=fpq1_contract.protections, premium=fpq1_contract.premium
        )

        a_insurer_name = "Assureur"
        fidelity_program_name = "FIDELITY"
        a_new_page = "<###NEW_PAGE###>"

        expected = (
            "Item 4. COVERAGE / PERILS: The perils covered by the insurance contract are those for which an amount "
            "of insurance, a deductible or an insurance premium is shown in the table below. Coverage is subject to "
            "the conditions set out in the insurance contract.\n"
            "PREMIUM\n"
            "\t Section A - CIVIL LIABILITY -\n"
            f"Property damage or bodily injury to another person - Amount of insurance: 1000000$ "
            f"{a_liability_coverage_premium}$\n"
            "\t Section B - DAMAGE TO INSURED VEHICLES\n"
            "Protection 1: All risks Included\n"
            "Protection 2: Risk of collision and rollover - Deductible 250$ per claim 2$\n"
            "Protection 3: All risks except collision or rollover - Deductible 250$ per claim 3$\n"
            "Protection 4: Specific risks Excluded\n"
            f"{a_new_page}\n"
            "Endorsement(s): Q.E.F. N 5a - Vehicles leased or under a contract of leasing Included\n"
            f"\t {fidelity_program_name} program\n"
            f"Q.E.F. N 41\n"
            f"- Change to deductibles (Section B): Included\n"
            f"This endorsement makes the following changes to the deductible amounts under Section B\n"
            f"\t - No deductible to pay in the case of a hit-and-run offence if a police report was issued.\n"
            f"\t - No deductible to pay in the case of a total loss.\n"
            f"\t - No deductible to pay in the case of a windshield repair except in the case of a windshield "
            f"replacement.\n "
            f"\t - Should you incur a loss that affects two or more insurance products insured at "
            f"{a_insurer_name}, you will have but one deductible to pay, the highest applicable.\n"
            "Premium due date: according to the agreed terms of payment\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAENContractWithEndorsementFAQ23A_whenGenerateItem4Text_thenReturnProperENText(self):
        a_liability_coverage_premium = 1
        a_b2_premium = 2
        a_b3_premium = 3
        a_faq_premium = 4
        protection_premiums = {
            "LiabilityCoveragePremium": a_liability_coverage_premium,
            "PropertyDamageB2CoveragePremium": a_b2_premium,
            "PropertyDamageB3CoveragePremium": a_b3_premium,
            "FAQ23ACoveragePremium": a_faq_premium,
        }
        a_premium = Premium(**protection_premiums)

        a_dict_of_protections = {
            "LiabilityCoverage": 1000000,
            "PropertyDamageB1Coverage": INCLUDE,
            "PropertyDamageB2Coverage": 250,
            "PropertyDamageB3Coverage": 250,
            "PropertyDamageB4Coverage": EXCLUDE,
            "FAQ23ACoverage": INCLUDE,
        }
        a_protections = Protections(protections=a_dict_of_protections)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=a_premium,
            financing=self.a_creditor,
            locale=self.a_en_locale,
        )

        actual = en.declaration_articles.generate_item_4_text(
            protections=fpq1_contract.protections, premium=fpq1_contract.premium
        )

        a_insurer_name = "Assureur"
        fidelity_program_name = "FIDELITY"
        a_new_page = "<###NEW_PAGE###>"

        expected = (
            "Item 4. COVERAGE / PERILS: The perils covered by the insurance contract are those for which an amount "
            "of insurance, a deductible or an insurance premium is shown in the table below. Coverage is subject to "
            "the conditions set out in the insurance contract.\n"
            "PREMIUM\n"
            "\t Section A - CIVIL LIABILITY -\n"
            f"Property damage or bodily injury to another person - Amount of insurance: 1000000$ "
            f"{a_liability_coverage_premium}$\n"
            "\t Section B - DAMAGE TO INSURED VEHICLES\n"
            "Protection 1: All risks Included\n"
            "Protection 2: Risk of collision and rollover - Deductible 250$ per claim 2$\n"
            "Protection 3: All risks except collision or rollover - Deductible 250$ per claim 3$\n"
            "Protection 4: Specific risks Excluded\n"
            f"{a_new_page}\n"
            "Endorsement(s): Q.E.F. N 23a - Vehicles under financing Included\n"
            f"\t {fidelity_program_name} program\n"
            f"Q.E.F. N 41\n"
            f"- Change to deductibles (Section B): Included\n"
            f"This endorsement makes the following changes to the deductible amounts under Section B\n"
            f"\t - No deductible to pay in the case of a hit-and-run offence if a police report was issued.\n"
            f"\t - No deductible to pay in the case of a total loss.\n"
            f"\t - No deductible to pay in the case of a windshield repair except in the case of a windshield "
            f"replacement.\n "
            f"\t - Should you incur a loss that affects two or more insurance products insured at "
            f"{a_insurer_name}, you will have but one deductible to pay, the highest applicable.\n"
            "Premium due date: according to the agreed terms of payment\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAENContractWithEndorsementFAQ27_whenGenerateItem4Text_thenReturnProperENText(self):
        a_liability_coverage_premium = 1
        a_b2_premium = 2
        a_b3_premium = 3
        a_faq_premium = 4
        protection_premiums = {
            "LiabilityCoveragePremium": a_liability_coverage_premium,
            "PropertyDamageB2CoveragePremium": a_b2_premium,
            "PropertyDamageB3CoveragePremium": a_b3_premium,
            "FAQ27CoveragePremium": a_faq_premium,
        }
        a_premium = Premium(**protection_premiums)

        a_dict_of_protections = {
            "LiabilityCoverage": 1000000,
            "PropertyDamageB1Coverage": INCLUDE,
            "PropertyDamageB2Coverage": 250,
            "PropertyDamageB3Coverage": 250,
            "PropertyDamageB4Coverage": EXCLUDE,
            "FAQ27Coverage": INCLUDE,
        }
        a_protections = Protections(protections=a_dict_of_protections)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=a_premium,
            financing=self.a_creditor,
            locale=self.a_en_locale,
        )

        actual = en.declaration_articles.generate_item_4_text(
            protections=fpq1_contract.protections, premium=fpq1_contract.premium
        )

        a_insurer_name = "Assureur"
        fidelity_program_name = "FIDELITY"
        a_new_page = "<###NEW_PAGE###>"

        expected = (
            "Item 4. COVERAGE / PERILS: The perils covered by the insurance contract are those for which an amount "
            "of insurance, a deductible or an insurance premium is shown in the table below. Coverage is subject to "
            "the conditions set out in the insurance contract.\n"
            "PREMIUM\n"
            "\t Section A - CIVIL LIABILITY -\n"
            f"Property damage or bodily injury to another person - Amount of insurance: 1000000$ "
            f"{a_liability_coverage_premium}$\n"
            "\t Section B - DAMAGE TO INSURED VEHICLES\n"
            "Protection 1: All risks Included\n"
            "Protection 2: Risk of collision and rollover - Deductible 250$ per claim 2$\n"
            "Protection 3: All risks except collision or rollover - Deductible 250$ per claim 3$\n"
            "Protection 4: Specific risks Excluded\n"
            f"{a_new_page}\n"
            "Endorsement(s): Q.E.F. N 27 - Civil liability resulting from damage caused to vehicles of "
            "which named insured is not owner (including vehicles provided by an employer) (Section A) - 75000$\n"
            "Vehicles included under described vehicles, trailers or utility trailers\n"
            "Section B2 - $250 deductible and section B3 - $50 deductible\n"
            f"\t {fidelity_program_name} program\n"
            f"Q.E.F. N 41\n"
            f"- Change to deductibles (Section B): Included\n"
            f"This endorsement makes the following changes to the deductible amounts under Section B\n"
            f"\t - No deductible to pay in the case of a hit-and-run offence if a police report was issued.\n"
            f"\t - No deductible to pay in the case of a total loss.\n"
            f"\t - No deductible to pay in the case of a windshield repair except in the case of a windshield "
            f"replacement.\n "
            f"\t - Should you incur a loss that affects two or more insurance products insured at "
            f"{a_insurer_name}, you will have but one deductible to pay, the highest applicable.\n"
            "Premium due date: according to the agreed terms of payment\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAENContractWithEndorsementFAQ27a_whenGenerateItem4Text_thenReturnProperENText(self):
        a_liability_coverage_premium = 1
        a_b2_premium = 2
        a_b3_premium = 3
        a_faq_premium = 4
        protection_premiums = {
            "LiabilityCoveragePremium": a_liability_coverage_premium,
            "PropertyDamageB2CoveragePremium": a_b2_premium,
            "PropertyDamageB3CoveragePremium": a_b3_premium,
            "FAQ27ACoveragePremium": a_faq_premium,
        }
        a_premium = Premium(**protection_premiums)

        a_dict_of_protections = {
            "LiabilityCoverage": 1000000,
            "PropertyDamageB1Coverage": INCLUDE,
            "PropertyDamageB2Coverage": 250,
            "PropertyDamageB3Coverage": 250,
            "PropertyDamageB4Coverage": EXCLUDE,
            "FAQ27ACoverage": INCLUDE,
        }
        a_protections = Protections(protections=a_dict_of_protections)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=a_premium,
            financing=self.a_creditor,
            locale=self.a_en_locale,
        )

        actual = en.declaration_articles.generate_item_4_text(
            protections=fpq1_contract.protections, premium=fpq1_contract.premium
        )

        a_insurer_name = "Assureur"
        fidelity_program_name = "FIDELITY"
        a_new_page = "<###NEW_PAGE###>"

        expected = (
            "Item 4. COVERAGE / PERILS: The perils covered by the insurance contract are those for which an amount "
            "of insurance, a deductible or an insurance premium is shown in the table below. Coverage is subject to "
            "the conditions set out in the insurance contract.\n"
            "PREMIUM\n"
            "\t Section A - CIVIL LIABILITY -\n"
            f"Property damage or bodily injury to another person - Amount of insurance: 1000000$ "
            f"{a_liability_coverage_premium}$\n"
            "\t Section B - DAMAGE TO INSURED VEHICLES\n"
            "Protection 1: All risks Included\n"
            "Protection 2: Risk of collision and rollover - Deductible 250$ per claim 2$\n"
            "Protection 3: All risks except collision or rollover - Deductible 250$ per claim 3$\n"
            "Protection 4: Specific risks Excluded\n"
            f"{a_new_page}\n"
            "Endorsement(s): Q.E.F. N 27a - Civil liability resulting from damage caused to vehicles of which named "
            "insured is not owner (including vehicles provided by an employer) (Section A) - 75000$\n"
            "Vehicles included under described vehicles, trailers or utility trailers\n"
            "Section B2 - $250 deductible and section B3 - $50 deductible\n"
            f"\t {fidelity_program_name} program\n"
            f"Q.E.F. N 41\n"
            f"- Change to deductibles (Section B): Included\n"
            f"This endorsement makes the following changes to the deductible amounts under Section B\n"
            f"\t - No deductible to pay in the case of a hit-and-run offence if a police report was issued.\n"
            f"\t - No deductible to pay in the case of a total loss.\n"
            f"\t - No deductible to pay in the case of a windshield repair except in the case of a windshield "
            f"replacement.\n "
            f"\t - Should you incur a loss that affects two or more insurance products insured at "
            f"{a_insurer_name}, you will have but one deductible to pay, the highest applicable.\n"
            "Premium due date: according to the agreed terms of payment\n"
        )

        self.assertEqual(expected, actual)



from ..endorsements.endorsement_header import generate_endorsement_page_header_text
from ...config import NEW_PAGE_TAG


@staticmethod
def generate_faq8_endorsement_text() -> str:
    endorsement_header_text = generate_endorsement_page_header_text()
    text = (
        f"{endorsement_header_text}"
        "Quebec Endorsement Form\n"
        "Q.E.F. N 8\n"
        "Deductible for property damage\n"
        "(Section A)\n"
        "The endorsement heading must be entered in the ''Declarations'' section of the insurance contract."
        "Details required for the endorsement may be entered in the ''Declarations'' section or in the endorsement"
        "itself, at the insurer's option.\n"
        "Name of insurer: \n"
        "Named insured: \n"
        "Endorsement to automobile insurance policy N:\n"
        "Effective date: This endorsement will apply from  at 12:01 A.M. "
        "standard time at the address of the named insured.\n"
        "Specified vehicle: This endorsement will apply only to the following described vehicle:\n"
        "(reference number shown in the ''Declarations'' section of the insurance contract)\n"
        "Endorsement description\n"
        "This endorsement changes Section A of the insurance contract by adding a deductible "
        "for property damage caused by the specified vehicle:\n"
        "\t A maximum deductible amount of  $ per loss.\n"
        "\t A maximum deductible amount of  $ per loss, when the vehicle is used to  .\n"
        "Agreement by named insured\n"
        "When the insurer pays an indemnity for property damage, the named insured agrees to reimburse "
        "the insurer for up to the deductible amount.\n"
        "Reimbursement must be made by the named insured as soon as the insurer pays the indemnity.\n"
        "Rights of insurer\n"
        "In respect of the deductible, the insurer will be entitled to:\n"
        "\t act as it wishes with regard to any investigation, transaction or settlement;\n"
        "\t authorize the named insured to enter into a transaction or a settlement with another "
        "person that has suffered property damage and indemnify that person. However, the total "
        "amount agreed upon further to the transaction or settlement must not exceed the deductible amount.\n"
        "All other conditions of the insurance contract remain the same.\n"
        f"{NEW_PAGE_TAG}\n"
    )

    return text



from ..endorsements.endorsement_header import generate_endorsement_page_header_text
from ...config import NEW_PAGE_TAG


@staticmethod
def generate_faq34_endorsement_text() -> str:
    endorsement_header_text = generate_endorsement_page_header_text()
    text = (
        f"{endorsement_header_text}"
        "Quebec Endorsement Form\n"
        "Q.E.F. N 34\n"
        "Accident benefits insurance\n"
        "The endorsement heading must be entered in the ''Declarations'' section of the insurance contract."
        "Details required for the endorsement may be entered in the ''Declarations'' section or in the endorsement"
        "itself, at the insurer's option.\n"
        "Name of insurer: \n"
        "Named insured: \n"
        "Endorsement to automobile insurance policy N:\n"
        "Effective date: This endorsement will apply from  at 12:01 A.M. "
        "standard time at the address of the named insured.\n"
        "Specified vehicle: This endorsement will apply only to the following described vehicle:\n"
        "(reference number shown in the ''Declarations'' section of the insurance contract)\n"
        "TABLE OF CONTENTS\n"
        "APPLICATION 2\n"
        "INSURED PERSONS 2\n"
        "APPLICABLE DIVISION, MAXIMUM AMOUNTS AND INSURANCE PREMIUM 2\n"
        "DESCRIPTION OF DIVISIONS 3\n"
        "EXCLUSIONS 5\n"
        "WHAT TO DO IN THE EVENT OF AN ACCIDENT 6\n"
        "AUTOPSY AND MEDICAL EXAMINATION 6\n"
        "TIME TO PAY BENEFITS 6\n"
        "RIGHTS OF INSURER IN RELATION TO PAYMENT OF BENEFITS OR EXPENSES 7\n"
        "DEFINITIONS 7\n"
        f"{NEW_PAGE_TAG}\n"
        f"{endorsement_header_text}"
        "ENDORSEMENT DESCRIPTION\n"
        "Depending on the applicable division, this endorsement provides for the payment "
        "of benefits when an accident causes bodily injury to an insured person.\n"
        "To determine which division applies, refer to the ''Applicable section, maximum amounts "
        "and insurance premium'' division of this endorsement.\n"
        "APPLICATION\n"
        "1. The motor vehicle must have been the direct cause of bodily injury, independently of "
        "any other cause.\n"
        "2. Benefits are payable per accident and per insured person. If there are several "
        "insured vehicles under the insurance contract, benefits may not be combined\n"
        "3. Only persons who comply with the conditions of the insurance contract and this "
        "endorsement may benefit from the coverage under this endorsement.\n"
        "INSURED PERSONS\n"
        "Under Division 1, ''insured person'' refers to the named insured, his or her spouse and "
        "any dependent children of either.\n"
        "Under Division 2, only the named insured is an ''insured person.''\n"
        "APPLICABLE DIVISION, MAXIMUM AMOUNTS AND INSURANCE PREMIUM\n"
        "Coverage will apply only for the divisions and subdivisions where a maximum amount or an "
        "insurance premium is shown in the table below, or entered specifically for this endorsement "
        "in the ''Declarations'' division of the insurance contract:\n"
        "DIVISION | SUBDIVISION | MAXIMUM | INSURANCE PREMIUM\n"
        "1 | A – Death benefits, B – Dismemberment benefits, C – Reimbursement of medical "
        "expenses | Principal sum $, Principal sum $, Maximum $ |  $,  $,  $"
        "2 Total disability benefits | Maximum  $, per week |  $|\n"
        "  |  | Total:|  $\n"
        f"{NEW_PAGE_TAG}\n"
        f"{endorsement_header_text}"
        "DESCRIPTION OF DIVISIONS\n"
        "Division 1\n"
        "Death and dismemberment benefits and reimbursement of medical expenses\n"
        "Subdivision A - Death benefits\n"
        "When an insured person dies within 12 months after the accident leading to his or her death, "
        "the following benefits will be paid under Subdivision A:\n"
        "\t If the named insured dies: 100% of the principal sum, plus 10% of the principal "
        "sum per dependent child upon the death of the named insured.\n"
        "\t If the spouse of the named insured dies:  % of the principal sum, plus 10% of "
        "the principal sum per dependent child upon the death of the spouse.\n"
        "\t If a dependent child dies:  % of the principal sum.\n"
        "Benefit payments will be made as follows:\n"
        "\t If the insured person dies leaving a spouse, the benefit will be payable to the spouse.\n"
        "\t If the insured person dies without leaving a spouse, the benefit will be payable "
        "to the dependents, in equal shares.\n"
        "\t If the insured person dies without leaving either a spouse or dependents, the benefit "
        "will be payable to the insured person's succession.\n"
        "\t If a dependent child dies, the benefit will be payable to the named insured.\n"
        "If, as a result of the same accident, an insured person should suffer dismemberment and then "
        "die, the benefits that have already been paid under Subdivision B will be deducted from the "
        "benefit payable under Subdivision A.\n"
        "Subdivision B – Dismemberment benefits\n"
        "When an insured person loses his or her sight or a limb within 12 months after "
        "the accident that caused the loss, the following benefits will be paid under Subdivision B:\n"
        "\t Loss of both hands: 100% of the principal sum;\n"
        "\t Loss of both feet: 100% of the principal sum;\n"
        "\t Loss of sight in both eyes: 100% of the principal sum;\n"
        "\t Loss of one hand and one foot: 100% of the principal sum;\n"
        "\t Loss of one hand and sight in one eye: 100% of the principal sum;\n"
        "\t Loss of one foot and sight of one eye: 100% of the principal sum;\n"
        "\t Loss of one arm: 75% of the principal sum;\n"
        "\t Loss of one leg: 75% of the principal sum;\n"
        "\t Loss of one hand: 50% of the principal sum;\n"
        "\t Loss of one foot: 50% of the principal sum;\n"
        "\t Loss of sight in one eye: 50% of the principal sum.\n"
        f"{NEW_PAGE_TAG}\n"
        f"{endorsement_header_text}"
        "Loss as used with reference to eye means total and irrecoverable loss of sight.\n"
        "Loss as used with reference to a limb means:\n"
        "\t For an arm: complete severance above the elbow.\n"
        "\t For a foot: complete severance through the knee, above the ankle or through the ankle.\n"
        "\t For a hand: complete severance through the elbow, above the wrist or through the wrist.\n"
        "\t For a leg: complete severance above the knee.\n"
        "Benefits provided under this subdivision for the same accident may not exceed 100% of "
        "the principal sum.\n"
        "Subdivision C –- Reimbursement of medical expenses\n"
        "Expenses reasonably incurred for the following services, care or treatments will be "
        "reimbursed under Division C:\n"
        "\t ambulance services;\n"
        "\t dental treatment;\n"
        "\t hospital care;\n"
        "\t medical care;\n"
        "\t professional nursing care;\n"
        "\t surgical treatment.\n"
        "The expenses must have been incurred within two years after the accident that caused them.\n"
        "The total amount of expenses reimbursed for the same accident may not exceed the maximum "
        "for this subsection.\n"
        "Exclusion\n"
        "Subdivision C excludes the portion of medical expenses covered by any other "
        "government or private insurance, unless the other insurance is similar to this coverage.\n"
        "Division 2\n"
        "Total disability benefits\n"
        "Division 2 provides for the payment of an income replacement benefit when the named insured becomes "
        "wholly and continuously disabled as a result of an accident. The income replacement indemnity "
        "will be calculated on a weekly basis and will be payable only in addition to the following amounts:\n"
        "\t amounts payable by the Société de l'assurance automobile du Québec under the Quebec "
        "Automobile Insurance Act; and\n"
        "\t amounts payable under the Act respecting industrial accidents and occupational "
        "disease and the Crime Victims Compensation Act.\n"
        "Conditions\n"
        "1. The named insured must be employed on the date of the accident. The named insured is "
        "deemed to be employed in either of the following situations:\n"
        "\t - he or she was actively engaged in an occupation or employment for wages or profit;\n"
        "\t - he or she was between 21 and 65 years of age and, in the 12 months preceding the accident, "
        "he or she was engaged in an occupation or employment for wages or profit for at least "
        "six months, whether continuously or not.\n"
        "2. The disability must appear within 20 days after the accident that caused it.\n"
        "3. The disability completely prevents the named insured from actively engaging in "
        "his or her occupation or employment."
        f"{NEW_PAGE_TAG}\n"
        f"{endorsement_header_text}"
        "Limitations\n"
        "For each accident:\n"
        "The named insured will not be entitled to benefits for the first seven days of disability.\n"
        "Once the seven days have elapsed, the named insured will be entitled to benefits "
        "for a maximum of   weeks.\n"
        "If the disability continues after the maximum number of weeks, benefits will "
        "continue to be payable for a further period of up to   weeks, if:\n"
        "\t continuation of the disability is duly certified; and\n"
        "\t the disability prevents the named insured from engaging in any occupation "
        "or employment for wages or profit on a permanent basis.\n"
        "The named insured may not receive benefits greater than the money value of "
        "any occupation or employment that he engaged in for wages or profit before the accident.\n"
        "If the named insured is covered under several insurance policies, benefits will be calculated as follows:\n"
        "Money value of all occupations and employment / Total benefits payable for the "
        "disability under all insurance policies X (times) Benefit payable under Division 2 of this endorsement\n"
        "EXCLUSIONS\n"
        "The following exclusions will apply:\n"
        "A. Bodily injury caused directly or indirectly by sickness or disease, unless the "
        "sickness or disease was contracted as a direct result of an accident covered by this endorsement.\n"
        "B. Bodily injury suffered by an insured person who committed or attempted to commit suicide, "
        "whether sane or insane, if this endorsement was not continuously in force for at least two years.\n"
        "C. Whether or not war is declared, bodily injury caused directly or indirectly by:\n"
        "\t bombardment;\n"
        "\t civil war;\n"
        "\t insurrection;\n"
        "\t invasion;\n"
        "\t military power;\n"
        "\t operation of armed forces during hostilities;\n"
        "\t rebellion;\n"
        "\t revolution; and\n"
        "\t usurped power.\n"
        f"{NEW_PAGE_TAG}\n"
        f"{endorsement_header_text}"
        "WHAT TO DO IN THE EVENT OF AN ACCIDENT\n"
        "Subdivision A - Death benefits\n"
        "In the event of an accident causing death, a claim must be submitted in writing to the insurer, "
        "accompanied by proof establishing, for example:\n"
        "\t death of the insured person;\n"
        "\t cause of death;\n"
        "\t rights of beneficiaries.\n"
        "Subdivisions B and C, and Division 2 - Dismemberment benefits, reimbursement of medical "
        "expenses and total disability benefits\n"
        "In the event of an accident covered under Subdivision B or C, or Section 2, the insured "
        "person or any other beneficiary must fulfil the obligations outlined below. They may do "
        "so either personally or through an intermediary.\n"
        "Obligations to fulfil:\n"
        "\t Within 30 days of learning of the accident, the person must inform the insurer in writing.\n"
        "\t Within 90 days of learning of the accident, the person must support the claim by providing the "
        "insurer with all the information that the insurer may reasonably expect as to the circumstances "
        "and extent of the accident.\n"
        "\t In the event of failure to satisfy the 30 or 90-day requirement, the person must demonstrate that "
        "he or she was unable to act within those time periods. Benefit payment may be made "
        "nonetheless if the person fulfils his or her obligations within one year after the accident.\n"
        "\t At the insurer's request, the person must provide a medical certificate confirming the following "
        "information:\n"
        "- the cause and nature of the bodily injury for which the claim is being made;\n"
        "- the length of total disability arising from such bodily injury.\n"
        "AUTOPSY AND MEDICAL EXAMINATION\n"
        "Subdivision A - Death benefits\n"
        "If an insured person dies, the insurer may have an autopsy performed, subject to the "
        "requirements set out in the Civil Code of Quebec.\n"
        "Subdivisions B and C, and Division 2 - Dismemberment benefits, reimbursement of medical "
        "expenses and total disability benefits\n"
        "The insurer is entitled to require that the insured person undergo a medical examination within "
        "a reasonable time period, if warranted by the nature of the disability or the loss.\n"
        "The insurer may have the insured person undergo a medical examination as often as may be "
        "reasonably required while the claim is pending.\n"
        "The insured person will be required to submit to such examination.\n"
        "TIME TO PAY BENEFITS\n"
        "Subdivision A - Death benefits\n"
        "The insurer must pay the benefits within 30 days after receipt of the required information and documents.\n"
        f"{NEW_PAGE_TAG}\n"
        f"{endorsement_header_text}"
        "Subdivisions B and C - Dismemberment benefits and reimbursement of medical expenses\n"
        "The insurer must pay benefits or reimburse medical expenses within 60 days after receipt of "
        "the required information and documents.\n"
        "Division 2 - Total disability benefits\n"
        "The insurer must pay the initial benefit within 30 days after receipt of the required information "
        "and documents. During the continuation of disability, subsequent payments will be made at 30-day "
        "intervals, subject to submission of the proof set out in the ''What to do in the event of an "
        "accident'' section of this endorsement.\n"
        "RIGHTS OF INSURER IN RELATION TO PAYMENT OF BENEFITS OR EXPENSES\n"
        "The insurer may decide to pay all or part of the benefits or expenses directly to either the "
        "beneficiary or the natural or legal persons providing the medical services, care or treatments. "
        "If any amount remains payable, the total amount of benefits or expenses already paid will then "
        "be deducted from the maximum under the appropriate subsection.\n"
        "Before paying benefits:\n"
        "\t the insurer may require a release from the person to whom it paid a benefit "
        "or reimbursed expenses.\n"
        "\t the insurer may require that it be subrogated to the beneficiary's recourses "
        "against the person responsible for the bodily injury, up to the amount of the benefit it has "
        "paid. This means that the beneficiary's recourses are transferred to the insurer.\n"
        "In addition, the payment of a benefit or the reimbursement of medical expenses will not "
        "constitute an admission of liability as regards the accident. Such payment may therefore not be "
        "set up against the insured person or the insurer in respect of civil liability.\n"
        "DEFINITIONS\n"
        "For the purposes of this endorsement, the following definitions will be in addition to or "
        "will replace the definitions contained in the insurance contract.\n"
        "Bodily injury: Any injury of a physical nature, including death.\n"
        "Dependent child:\n"
        "\t Any child under 18 years of age for whom the named insured or the named "
        "insured's spouse has legal and actual custody;\n"
        "\t Any child 18 years of age or older whose domicile is the same as that of "
        "the named insured and who is entirely dependent on the named insured and/or the named "
        "insured's spouse due to a mental or physical disability.\n"
        f"{NEW_PAGE_TAG}\n"
        f"{endorsement_header_text}"
        "Dependents: This expression refers to the following persons:\n"
        "\t any dependent child; and\n"
        "\t the father or mother of the deceased insured person, if the following conditions are met:\n"
        "- the father or mother had the same domicile as the insured person on the date of the accident; and\n"
        "- the father or mother was principally dependent on the insured person for financial support up "
        "to the date of the accident.\n"
        "Named insured:\n"
        "\t The expression ''named insured'' will refer to any person named in Item 1, ''Declarations'' "
        "of the insurance contract;\n"
        "\t If the person named in Item 1 is a legal person, partnership or association, the "
        "expression will refer to each of its employees, shareholders, members or partners who "
        "usually use a described vehicle for which an insurance premium or a maximum amount "
        "is entered specifically for this endorsement (see Item 4, ''Declarations'' of the insurance contract).\n"
        "All other conditions of the insurance contract remain the same.\n"
        f"{NEW_PAGE_TAG}\n"
    )

    return text



import re

from ..general_header import generate_general_pages_header_text
from ...config import (
    FAKE_INSURER_ADDRESS,
    FAKE_INSURER_NAME,
    FAKE_INSURER_WEBSITE,
    NEW_PAGE_TAG,
    FAKE_FIDELITY_PROGRAM_NAME_EN,
    FAKE_ROADSIDE_PROGRAM_NAME,
)
from ....domain.premium import Premium
from ....domain.protections import Protections
from ....domain.tools import change_to_snake_case

all_property_damage_protection = [
    "PropertyDamageB1Coverage",
    "PropertyDamageB2Coverage",
    "PropertyDamageB3Coverage",
    "PropertyDamageB4Coverage",
]
endorsement_with_possible_premium = ["FAQ20", "FAQ20A", "FAQ33", "FAQ34", "FAQ34AB", "FAQ43"]
endorsement_without_premium = ["FAQ2", "FAQ5A", "FAQ23A", "FAQ27", "FAQ27A"]

# Base property damage text that we use to dynamically append the rest of the text base on the protections.
property_damage_b1_coverage_base_text = "Protection 1: All risks"
property_damage_b2_coverage_base_text = "Protection 2: Risk of collision and rollover"
property_damage_b3_coverage_base_text = "Protection 3: All risks except collision or rollover"
property_damage_b4_coverage_base_text = "Protection 4: Specific risks"

# Template for when the protection specified a deductible and a premium amount
property_damage_b1_coverage_deductible_template = "- Deductible {}$ per claim {}$"
property_damage_b2_coverage_deductible_template = "- Deductible {}$ per claim {}$"
property_damage_b3_coverage_deductible_template = "- Deductible {}$ per claim {}$"
property_damage_b4_coverage_deductible_template = "- Deductible {}$ per claim {}$"


def generate_declaration_header_text(policy_number: str) -> str:
    """
    Function to generate the declaration header text.

    policy_number (str): The policy number.

    Return:
        The declaration header text.
    """
    general_header_text = generate_general_pages_header_text(policy_number=policy_number)
    header = f"{general_header_text}" "DECLARATIONS\n" "ISSUE of your new insurance policy\n"
    return header


def generate_item_1_text(insuree_name: str, insuree_address: str) -> str:
    """
    Function to generate the item 1 text for the declaration.

    insuree_name (str): The name of the insuree.
    insuree_address (str): The address of the insuree.

    Return:
        The item 1 text.
    """
    text = (
        "Item 1. NAME AND ADDRESS OF THE NAMED INSURED*\n"
        f"{insuree_name}\n"
        f"{insuree_address}\n"
        "*The described vehicle is and will be mainly used, stored and parked in the town/city and province shown "
        "in Item 1. If not, the client or the named insured must so declare.\n"
    )
    return text


def generate_item_2_text(contract_start_date: str, contract_end_date: str) -> str:
    """
    Function to generate the item 2 text for the declaration.

    contract_start_date (str): The start date of the contract.
    contract_end_date (str): The end date of the contract.

    Return:
        The item 2 text.
    """
    text = (
        "Item 2. CONTRACT PERIOD\n"
        f"FROM: {contract_start_date}* TO: {contract_end_date}* EXCLUSIVELY\n"
        "*at 12:01 A.M. standard time at the address of the named insured.\n"
    )
    return text


def generate_item_3_text(
    formatted_vehicle_complete_details: str,
    car_ownership: str,
    vehicle_purchase_condition: str,
    car_with_financing: str,
    financing_details: str,
) -> str:
    """
    Function to generate the item 3 text for the declaration.

    formatted_vehicle_complete_details (str): The vehicle complete details in the proper format.
    car_ownership (str): The type of car ownership in the locale language (e.g. Bought/rented).
    vehicle_purchase_condition (str): The condition of the vehicle at purchase in the locale language (e.g. New)
    car_with_financing (Union[None, str]) The type of financing, if any. If None, it means no financing.
    financing_details (str): Financing details, if any.

    Return:
        The item 3 text.
    """
    text = (
        "Item 3. PARTICULARS OF THE DESCRIBED VEHICLE 1\n"
        f"{formatted_vehicle_complete_details} - {car_ownership} {vehicle_purchase_condition}\n"
    )

    if car_with_financing == "creditor":
        text += (
            "Creditor entitled to Chapter B benefits, according to his interest - Subject to the "
            "Q.E.F. endorsement N 23a\n"
            f"{financing_details}\n"
        )
    elif car_with_financing == "lessor":
        text += (
            "Lessor entitled to Chapter B benefits, according to his interest - Subject to the "
            "Q.E.F. endorsement N 5a\n"
            f"{financing_details}\n"
        )

    return text


# Endorsements with premium <


def generate_faq_20_text(faq_premium: str) -> str:
    """
    Function to generate FAQ20 text with a premium amount
    """
    text = (
        f"Q.E.F. N 20 - Travel Expense (Extended Form) (Chapter B) {faq_premium}$\n"
        "Travel expenses 50$ per day, total limit of 1500$ per claim\n"
        "Other covered expenses during a trip - 750$\n"
    )
    return text


def generate_faq_20a_text(faq_premium: str) -> str:
    """
    Function to generate FAQ20A text with a premium amount
    """
    text = (
        f"Q.E.F. N 20a - Travel Expense (Extended Form) (Chapter B) {faq_premium}$\n"
        "Travel expenses 50$ per day, total limit of 1500$ per claim\n"
        "Other covered expenses during a trip - 750$\n"
    )
    return text


def generate_faq_33_text(faq_premium: str) -> str:
    """
    Function to generate FAQ33 text with a premium amount
    """
    text = (
        "Q.E.F. N 33 - Insurance for roadside assistance expenses\n"
        f"{FAKE_ROADSIDE_PROGRAM_NAME} Roadside Assistance Program, available 24 hours a day, 7 days a week\n"
        "Maximum radius 50 km -\n"
        "In the event of a breakdown, the towing distance is increased to 100 km in the Laurentian Park, "
        "150 km in La Vérendrye Park and 100 km in Gaspésie National Park\n"
        f"Maximum number of events per insurance year 4 {faq_premium}$\n"
        "Limit per breakdown service 60$(1) - For a towing or a stuck vehicle, this limit is increased to 100$(1)\n"
        "Limit per policy year 400$(1)\n"
        "(1) These limits apply only when, on an exceptional basis, towing or breakdown service is not available "
        "through the Roadside Assistance Program\n"
        "See your brochure for all additional services available to you\n"
    )
    return text


def generate_faq_34ab_text(faq_premium: str) -> str:
    """
    Function to generate FAQ34AB text with a premium amount
    """
    text = (
        f"Q.E.F. N 34ab - Accident benefits insurance: {faq_premium}$\n"
        "Division 1: Subdivision A and B - Death and dismemberment benefits: 20000$ principal sum\n"
        "Subdivision C - Reimbursement of medical expenses: 2000$ per person\n"
        "Division 2: Total disability benefits: Not covered\n"
    )
    return text


def generate_faq_34_text(faq_premium: str) -> str:
    """
    Function to generate FAQ34 text with a premium amount
    """
    text = (
        f"Q.E.F. N 34 - Accident benefits insurance: {faq_premium}$\n"
        "Division 1: Subdivision A and B - Death and dismemberment benefits: 20000$ principal sum\n"
        "Subdivision C - Reimbursement of medical expenses: 2000$ per person\n"
        "Division 2: Total disability benefits: Not covered\n"
    )
    return text


def generate_faq_43_text(faq_premium: str) -> str:
    """
    Function to generate FAQ43 text with a premium amount.

    25% of the premium is the FAQ43A and 75% of the premium is the FAQ43E.
    """

    text = (
        f"Q.E.F. N 43a - Change to indemnity (Section B) - Partial loss - New parts "
        f"{round(int(faq_premium) * 0.25)}$\n"
        f"Q.E.F. N 43e - Change to indemnity (Section B) - Total loss - Replacement cost "
        f"{round(int(faq_premium) * 0.75)}$\n"
    )
    return text


# Endorsements with premium >


# Endorsements with no premium <
def generate_faq_2_text() -> str:
    """
    Function to generate FAQ2 text.
    """

    text = (
        "Q.E.F. N 2 - Vehicles of which named insured is not owner and when driven by named drivers "
        "(Section A) Included\n"
    )
    return text


def generate_faq_5a_text() -> str:
    """
    Function to generate FAQ5a text.
    """

    text = "Q.E.F. N 5a - Vehicles leased or under a contract of leasing Included\n"
    return text


def generate_faq_23a_text() -> str:
    """
    Function to generate FAQ23a text.
    """

    text = "Q.E.F. N 23a - Vehicles under financing Included\n"
    return text


def generate_faq_27_text() -> str:
    """
    Function to generate FAQ27 text.
    """

    text = (
        "Q.E.F. N 27 - Civil liability resulting from damage caused to vehicles of which named insured is not owner "
        "(including vehicles provided by an employer) (Section A) - 75000$\n"
        "Vehicles included under described vehicles, trailers or utility trailers\n"
        "Section B2 - $250 deductible and section B3 - $50 deductible\n"
    )
    return text


def generate_faq_27a_text() -> str:
    """
    Function to generate FAQ27a text.
    """

    text = (
        "Q.E.F. N 27a - Civil liability resulting from damage caused to vehicles of which named insured is not owner "
        "(including vehicles provided by an employer) (Section A) - 75000$\n"
        "Vehicles included under described vehicles, trailers or utility trailers\n"
        "Section B2 - $250 deductible and section B3 - $50 deductible\n"
    )
    return text


# Endorsements with no premium >
def generate_item_4_text(protections: Protections, premium: Premium) -> str:
    """
    Function to generate the item 4 text for the declaration.

    protections (Protections): The protections.
    premium (Premium): The premiums.

    Return:
        The item 4 text.
    """

    text = (
        "Item 4. COVERAGE / PERILS: The perils covered by the insurance contract are those for which an amount "
        "of insurance, a deductible or an insurance premium is shown in the table below. Coverage is subject to "
        "the conditions set out in the insurance contract.\n"
        "PREMIUM\n"
        "\t Section A - CIVIL LIABILITY -\n"
        f"Property damage or bodily injury to another person - Amount of insurance: "
        f"{protections.get('LiabilityCoverage')}$ {premium.get_premium('liability_coverage_premium')}$\n"
        f"\t Section B - DAMAGE TO INSURED VEHICLES\n"
    )

    # Chapter B text
    protection_with_deductible = protections.property_damage_protections()

    # We construct the text to use using template filling.
    for protection_name in all_property_damage_protection:

        # To extract the proper property damage text, we use the protection damage name written in CamelCase
        # and convert it into snake_case.
        snake_case_protection_name = change_to_snake_case(protection_name)
        base_text = eval(f"{snake_case_protection_name}_base_text")

        if protection_name in protection_with_deductible:
            # If the protection is one with a deductible, thus it need to be written in the protection text and include
            # a premium amount.
            protection_text = eval(f"{snake_case_protection_name}_deductible_template")
            protection_text = protection_text.format(
                protections.get(protection_name), premium.get_premium(f"{protection_name}Premium")
            )
        else:
            # The protection is either 'Include' or 'Exclude' (written in the locale language).
            if protections.is_protected(protection_name):
                # If the protection name is protected, it means it is 'include' (but written in the local language).
                protection_text = "Included"
            else:
                # If the protection name is not protected, it means it is 'exclude' (but written in the local language).
                protection_text = "Excluded"

        text += base_text + " " + protection_text + "\n"

    # We arbitrarily split the item 4 after the chapter B
    text += NEW_PAGE_TAG + "\n"

    # Endorsement text
    endorsement_protections = protections.endorsement_protections()
    for endorsement in endorsement_protections:
        if endorsement.strip("Coverage") in endorsement_with_possible_premium and protections.is_protected(
            f"{endorsement}"
        ):
            text += "Endorsement(s): "
            faq_premium = premium.get_premium(f"{endorsement}Premium")
            endorsement_number = re.findall(r'[0-9]+[AB]*', endorsement)[0].lower()
            text += eval(f"generate_faq_{endorsement_number}_text(faq_premium={faq_premium})")

        elif endorsement.strip("Coverage") in endorsement_without_premium and protections.is_protected(
            f"{endorsement}"
        ):
            text += "Endorsement(s): "
            endorsement_number = re.findall(r'[0-9]+[AB]*', endorsement)[0].lower()
            text += eval(f"generate_faq_{endorsement_number}_text()")

    text += (
        f"\t {FAKE_FIDELITY_PROGRAM_NAME_EN} program\n"
        f"Q.E.F. N 41\n"
        f"- Change to deductibles (Section B): Included\n"
        f"This endorsement makes the following changes to the deductible amounts under Section B\n"
        f"\t - No deductible to pay in the case of a hit-and-run offence if a police report was issued.\n"
        f"\t - No deductible to pay in the case of a total loss.\n"
        f"\t - No deductible to pay in the case of a windshield repair except in the case of a windshield "
        f"replacement.\n "
        f"\t - Should you incur a loss that affects two or more insurance products insured at "
        f"{FAKE_INSURER_NAME}, you will have but one deductible to pay, the highest applicable.\n"
    )

    text += "Premium due date: according to the agreed terms of payment\n"

    return text


def generate_premium_ticket(contract_starting_date: str, premium: Premium) -> str:
    text = (
        f"Prepared by the Insurer in Quebec City on {contract_starting_date}\n"
        f"Sherlock Holmes\n"
        f"President and chief executive officer\n"
        f"This is not an invoice. Please refer to the billing documents"
        f"for information on the amounts payable or receivable\n"
        f"PREMIUM VEHICLE 1\n"
        f"Class of insurance:\n"
        f"AUTOMOBILE\n"
        f"ANNUAL PREMIUM\n"
        f"{premium.total}$ (excluding tax)\n"
        f"YOUR NEW 12-MONTH POLICY\n"
        f"MONTANT\n"
        f"{premium.total}$ (excluding tax)\n"
        f"TAX\n"
        f"{premium.total * 0.09}$\n"
    )  # The Quebec taxes is 9%
    return text


def generate_item_6_text(
    insuree_name: str, insuree_birth_date: str, has_suspension: bool, number_of_claims_past_years: int
) -> str:
    """
    Function to generate the item 6 text for the declaration.

    insuree_name (str): The name of the insuree.
    insuree_birth_date (str): The birthdate of the insuree.
    has_suspension (bool): Either or not the insuree has a suspension.
    number_of_claims_past_years (int): The number of claims the insuree has.

    Return:
        The item 6 text.
    """
    text = (
        "Item 6. IMPORTANT STATEMENTS FOR ANALYZING THE RISK\n"
        f"\t Main driver: {insuree_name} Date of birth: {insuree_birth_date}\n"
    )

    if has_suspension:
        text += "\t At least one revocation or suspension of licence has been reported over the past three years\n"
    else:
        text += "\t No revocation or suspension of licence has been reported over the past three years\n"

    if number_of_claims_past_years > 0:
        text += f"\t {number_of_claims_past_years} loss or claim has been reported over the past five years\n"
    else:
        text += "\t No loss or claim has been reported over the past five years\n"

    return text


def generate_item_7_text(association_rebate: bool, payment_method: str) -> str:
    """
    Function to generate the item 7 text for the declaration.

    association_rebate (bool): Either or not the insuree has an association rebate.
    payment_method (str): The payment method.

    Return:
        The item 7 text.
    """

    text = "Item 7. INFORMATION FOR THE NAMED INSURED\n"

    if association_rebate or payment_method == "pre-authorize":
        text += "\t The premium takes into account the following discounts:\n"
        if association_rebate:
            text += "- As a member/customer of an association, you are eligible for the following discount: 10%.\n"
        if payment_method == "pre-authorize":
            text += "- 2% for choosing Pre-authorized payment as your premium payment method\n"

    text += (
        "\t Consent:If authorization has been granted for us to obtain credit information for the purposes of "
        "issuing this contract, please note that the information obtained will be updated and used for renewing "
        "your contract and for processing your property and casualty insurance files. This consent can be revoked "
        "at any time by notifying us in writing.\n"
    )

    return text


def generate_divulgation_text() -> str:
    """
    Function to generate the divulgation text for the declaration.

    Return:
        The divulgation text.
    """

    text = (
        "DISCLOSURE\n\n"
        "NOTICE CONCERNING THE PROTECTION OF PERSONAL INFORMATION\n"
        f"{FAKE_INSURER_NAME} protects the confidentiality of your personal information.\n"
        f"PURPOSE OF YOUR FILE\n"
        f"We collect and use your personal information to manage your Property and Casualty insurance file.\n"
        f"SECURITY\n"
        f"Your personal information is stored at our offices and protected by high security measures. Only our "
        f"employees, mandataries, distribution partners and service providers may access your personal information, "
        f"and solely when such access is required to perform their duties, carry out their mandate or fulfill "
        f"their service contract.\n"
        f"{FAKE_INSURER_NAME} may do business with one or more service providers based outside of Quebec. "
        f"It is therefore possible that some of your personal information held by "
        f"{FAKE_INSURER_NAME} may be stored outside of Quebec and governed by the laws of foreign "
        f"countries or states.\n"
        f"ACCESS AND CORRECTION\n"
        f"To access your file or make a correction to it, send your request in writing to the following address:\n"
        f"{FAKE_INSURER_NAME}\n"
        f"c/o Access to Information Service\n"
        f"{FAKE_INSURER_ADDRESS}\n"
        f"SERVICE OFFERING\n"
        f"{FAKE_INSURER_NAME}, its subsidiaries and their authorized representatives may use your personal "
        f"information to inform you of products and services that may be of interest to you. If, however, "
        f"you do not wish to receive this type of information, please write to us at the address above.\n"
        f"For more information about our personal information protection practices, refer to our personal "
        f"information protection statement at {FAKE_INSURER_WEBSITE}/en/personal-information-protection.\n"
        f"Your agent offers {FAKE_INSURER_NAME} products on an exclusive basis and does not receive any commission.\n"
    )
    return text



from ..endorsements.endorsement_header import generate_endorsement_page_header_text
from ...config import NEW_PAGE_TAG


@staticmethod
def generate_faq2_endorsement_text() -> str:
    endorsement_header_text = generate_endorsement_page_header_text()
    text = (
        f"{endorsement_header_text}"
        "Formulaire d'avenant du Québec\n"
        "F.A.Q. N 2\n"
        "Conduite de véhicules dont l'assuré désigné n'est pas propriétaire par des conducteurs désignés\n"
        "(Chapitre A)\n"
        "Le titre de l'avenant doit être écrit à la section « Conditions particulières » du contrat "
        "d'assurance. Quant aux informations requises dans l'avenant, elles peuvent être écrites à "
        "cette section ou dans l'avenant même, au choix de l'assureur.\n"
        "Nom de l'assureur : \n"
        "Nom de l'assuré désigné : \n"
        "Avenant à la police d'assurance automobile N :\n"
        "Date de prise d'effet : cet avenant s'applique à partir du à 0 h 01, heure normale "
        "à l'adresse de l'assuré désigné.\n"
        "Prime d'assurance additionnelle à payer :\n"
        "\t Montants à payer :  \n"
        "\t Date limite pour payer :  \n"
        "Description de l'avenant\n"
        "Cet avenant étend les garanties du chapitre A du contrat d'assurance en ajoutant le paragraphe "
        "suivant à l'article 2 intitulé « Véhicules assurés » :\n"
        "« tout véhicule assimilable à   et conduit, au moment du "
        "sinistre, par l'une des personnes suivantes :\n"
        "NOM | ÂGE | LIEN AVEC L'ASSURÉ DÉSIGNÉ\n"
        "1. | | \n"
        "2. | | \n"
        "3. | | \n"
        "4. | | \n"
        "Pour que ce véhicule soit considéré comme un « véhicule assuré » au chapitre A, "
        "les conditions suivantes doivent être respectées :\n"
        "1. Au moment du sinistre, le véhicule n'est pas conduit dans le cadre d'une "
        "activité professionnelle de garagiste.\n"
        "2. Le véhicule n'a pas comme propriétaire ou usager fréquent les personnes suivantes :\n"
        "\t l'assuré désigné ou toute personne ayant le même domicile que lui;\n"
        "\t toute personne désignée dans le tableau ci-dessus ou toute personne ayant le même "
        "domicile qu'elle.\n"
        f"{NEW_PAGE_TAG}\n"
        f"{endorsement_header_text}"
        "3. Le véhicule n'est pas fourni par un employeur :\n"
        "\t de l'assuré désigné ou de toute personne ayant le même domicile que lui;\n"
        "\t d'une personne désignée dans le tableau ci-dessus ou de toute personne ayant le même "
        "domicile qu'elle.\n"
        "4. Le véhicule n'est pas affecté :\n"
        "\t à l'usage de taxi, d'autobus ou d'autocar;\n"
        "\t à la livraison commerciale.»\n"
        "Toutes les autres conditions du contrat d'assurance restent les mêmes.\n"
        f"{NEW_PAGE_TAG}\n"
    )

    return text



from .declaration_articles.items import (
    generate_declaration_header_text,
    generate_item_1_text,
    generate_item_2_text,
    generate_item_3_text,
    generate_item_4_text,
    generate_item_6_text,
    generate_item_7_text,
    generate_divulgation_text,
    generate_premium_ticket,
)
from ..config import NEW_PAGE_TAG


def generate_declaration_text(self) -> str:
    header = generate_declaration_header_text(policy_number=self.policy_number)

    item_1 = generate_item_1_text(insuree_name=self.insuree.name, insuree_address=self.insuree.address)

    item_2 = generate_item_2_text(
        contract_start_date=self.contract_start_date, contract_end_date=self.contract_end_date
    )

    item_3 = generate_item_3_text(
        formatted_vehicle_complete_details=self.vehicle.format_vehicle_complete_details(with_text="Numéro de série"),
        car_ownership=self.car_ownership(),
        vehicle_purchase_condition=self.vehicle.purchase_condition,
        car_with_financing=self.car_with_financing(),
        financing_details=self.financing,
    )

    item_4 = generate_item_4_text(protections=self.protections, premium=self.premium)
    premium_ticket = generate_premium_ticket(contract_starting_date=str(self.contract_start_date), premium=self.premium)

    item_6 = generate_item_6_text(
        insuree_name=self.insuree.name,
        insuree_birth_date=self.insuree.date_of_birth,
        has_suspension=self.insuree.suspension,
        number_of_claims_past_years=self.insuree.number_of_claims_past_years,
    )

    item_7 = generate_item_7_text(
        association_rebate=self.insuree.association_rebate, payment_method=self.payment_method
    )

    divulgation = generate_divulgation_text()

    text = (
        header
        + item_1
        + item_2
        + item_3
        + item_4
        + NEW_PAGE_TAG
        + "\n"
        + premium_ticket
        + NEW_PAGE_TAG
        + "\n"
        + item_6
        + item_7
        + NEW_PAGE_TAG
        + "\n"
        + divulgation
        + NEW_PAGE_TAG
        + "\n"
    )

    return text



from unittest import TestCase

from risc_generator import SexFaker


class SexFakerTests(TestCase):
    n = 250000
    delta = 0.01  # A delta of 1%, meaning we accept a 1% deviation from the expected probabilities

    def setUp(self) -> None:
        self.a_fr_locale = "fr_CA"
        self.a_en_locale = "en_CA"

    def test_whenSexFakerEn_thenAreInDeltaExpectedDefaultProb(
        self,
    ):
        sex_faker = SexFaker(locale=self.a_en_locale)

        sex = [sex_faker.gender() == "male" for _ in range(self.n)]

        actual = sum(sex) / self.n  # Prob of been a male driver
        expected = 0.52  # 2860709 / 5546433

        self.assertAlmostEqual(actual, expected, delta=self.delta)

    def test_whenSexFakerEn_thenAreInDeltaExpectedUserProb(
        self,
    ):
        sex_faker = SexFaker(locale=self.a_en_locale, population_of_drivers=1000000, number_of_male_drivers=100000)

        sex = [sex_faker.gender() == "male" for _ in range(self.n)]

        actual = sum(sex) / self.n  # Prob of been a male driver
        expected = 0.10  # (50000 + 50000) / 1000000

        self.assertAlmostEqual(actual, expected, delta=self.delta)

    def test_whenSexFakerFr_thenAreInDeltaExpectedDefaultProb(
        self,
    ):
        sex_faker = SexFaker(locale=self.a_fr_locale)

        sex = [sex_faker.gender() == "homme" for _ in range(self.n)]

        actual = sum(sex) / self.n  # Prob of been a male driver
        expected = 0.52  # 2860709 / 5546433

        self.assertAlmostEqual(actual, expected, delta=self.delta)

    def test_whenSexFakerFr_thenAreInDeltaExpectedUserProb(
        self,
    ):
        sex_faker = SexFaker(locale=self.a_fr_locale, population_of_drivers=1000000, number_of_male_drivers=100000)

        sex = [sex_faker.gender() == "homme" for _ in range(self.n)]

        actual = sum(sex) / self.n  # Prob of been a male driver
        expected = 0.10  # (50000 + 50000) / 1000000

        self.assertAlmostEqual(actual, expected, delta=self.delta)



import random
import string


def is_valid_vehicle_year(vehicle_year: int, contract_year: int) -> bool:
    """
    Function to validate if a vehicle year respect the 5 years gap for the FAQ43 endorsements.

    vehicle_year (int): Year of the vehicle.
    contract_year (int): Year of the contract.

    Return:
        Either or not the vehicle is valid.
    """
    return contract_year - vehicle_year <= 5


def generate_vin() -> str:
    """
    Generate a VIN number that DOES NOT respect proper VIN specs. It generates a random selection of 17 characters as
    the VIN, which is not a valid VIN. The characters are number (0 to 9) and ascii capitalize letters (A-Z)
    (no special characters).
    """

    return "".join(random.choices(string.digits + string.ascii_uppercase, k=17))



from unittest import TestCase

from risc_generator import PaymentMethodFaker


class PaymentMethodFakerTests(TestCase):
    n = 250000
    delta = 0.05  # A delta of 0.05, meaning we accept a 5% deviation from the expected probabilities

    def setUp(self) -> None:
        self.payment_method_faker = PaymentMethodFaker(locale="en_CA")

    def test_whenPaymentMethodFaker_thenAreInDeltaExpectedPreAuthorizeProb(
        self,
    ):
        payment_method = [self.payment_method_faker.payment_method() == "pre-authorize" for _ in range(self.n)]

        actual = sum(payment_method) / self.n  # Prob of having a pre-authorize payment method
        expected = 0.95

        self.assertAlmostEqual(actual, expected, delta=self.delta)



from .general_header import generate_general_pages_header_text
from ..config import GENERAL_FAKE_PHONE_NUMBER_1_800, NEW_PAGE_TAG, GENERAL_FAKE_PHONE_NUMBER


def generate_cancellation_of_policy_text(self):
    general_header_text = generate_general_pages_header_text(policy_number=self.policy_number)

    text = (
        f"{general_header_text}"
        "RÉSILIATION DE LA POLICE\n"
        "Ne pas utiliser cette formule si votre police couvre plusieurs "
        "véhicules et qu'un seul doit être résilié. "
        "Veuillez communiquer avec le service à la clientèle au numéro inscrit ci-dessous.\n"
        f"AVANT DE RÉSILIER communiquez avec le service à la clientèle au {GENERAL_FAKE_PHONE_NUMBER} ou "
        f"{GENERAL_FAKE_PHONE_NUMBER_1_800}; ceci pourrait vous éviter de perdre certains avantages.\n"
        f"Je demande la résiliation complète de la police numéro {self.policy_number}, "
        "de ses avenants, de ses renouvellements et s'il y a lieu, le remboursement de la prime "
        "non acquise à compter du : ______________      ______________      ______________\n"
        "(Année) (Mois) (Jour)\n"
        "Signature :  ____________________________________________________________   "
        "Date : _____________     _____________     _____________\n"
        "(Assuré) (Année) (Mois) (Jour)\n"
        "Signature :  ____________________________________________________________   "
        "Date : _____________     _____________     _____________\n"
        "(Assuré additionnel) (Année) (Mois) (Jour)\n"
        "Signature :  ____________________________________________________________   "
        "Date : _____________     _____________     _____________\n "
        "(Assuré additionnel) (Année) (Mois) (Jour)\n"
        "Raison de l'annulation :  _____________________________________________  "
        "Nouvel assureur :  ________________________________________\n"
        f"{NEW_PAGE_TAG}\n"
    )

    return text



import datetime
import re
from unittest.mock import patch

from risc_generator import FPQ1Contract
from risc_generator.content import fr, en
from tests.domain.contract.contract_base import ContractTestBase


class ContractDeclarationTest(ContractTestBase):
    def test_givenAFRContract_whenGenerateDeclarationText_thenReturnProperFRText(self):
        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=self.a_protections,
            premium=self.a_premium,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual_declaration_text = fpq1_contract.generate_declaration_text()

        actual_number_of_pages = len(re.findall("<###NEW_PAGE###>", actual_declaration_text))

        expected = 5

        self.assertEqual(expected, actual_number_of_pages)

    def test_givenAENContract_whenGenerateDeclarationText_thenReturnProperENText(self):
        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=self.a_protections,
            premium=self.a_premium,
            financing=self.a_creditor,
            locale=self.a_en_locale,
        )

        actual_declaration_text = fpq1_contract.generate_declaration_text()

        actual_number_of_pages = len(re.findall("<###NEW_PAGE###>", actual_declaration_text))

        expected = 5

        self.assertEqual(expected, actual_number_of_pages)

    @patch("risc_generator.contract.Faker")
    def test_givenAFRContract_whenGeneratePremiumTicket_thenReturnProperFRText(self, date_faker_mock):
        a_year = 2022
        a_month = 2
        a_day = 1

        date_faker_mock().date_between.return_value = datetime.date(year=a_year, month=a_month, day=a_day)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=self.a_protections,
            premium=self.a_premium,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fr.declaration.generate_premium_ticket(
            contract_starting_date=fpq1_contract.formatted_text_date(), premium=self.a_premium
        )
        expected = (
            "Fait par l'Assureur à Québec le 1 février 2022\n"
            "Sherlock Holmes\n"
            "Président et chef de la direction\n"
            "Ceci n'est pas une facture, veuillez vous référer aux documents de la facturation pour "
            "les montants à payer ou à recevoir\n"
            "PRIME DU VÉHICULE 1\n"
            "Catégorie d'assurance :\n"
            "AUTOMOBILE\n"
            "PRIME ANNUELLE\n"
            "7$ (excluant taxe)\n"
            "VOTRE NOUVELLE POLICE 12 MOIS\n"
            "MONTANT\n"
            "7$ (excluant taxe)\n"
            "TAXE\n"
            "0.63$\n"
        )

        self.assertEqual(expected, actual)

    @patch("risc_generator.contract.Faker")
    def test_givenAENContract_whenGeneratePremiumTicket_thenReturnProperENText(self, date_faker_mock):
        a_year = 2022
        a_month = 2
        a_day = 1

        date_faker_mock().date_between.return_value = datetime.date(year=a_year, month=a_month, day=a_day)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=self.a_protections,
            premium=self.a_premium,
            financing=self.a_creditor,
            locale=self.a_en_locale,
        )

        actual = en.declaration.generate_premium_ticket(
            contract_starting_date=fpq1_contract.formatted_text_date(), premium=self.a_premium
        )
        expected = (
            "Prepared by the Insurer in Quebec City on February 1, 2022\n"
            "Sherlock Holmes\n"
            "President and chief executive officer\n"
            "This is not an invoice. Please refer to the billing documents"
            "for information on the amounts payable or receivable\n"
            "PREMIUM VEHICLE 1\n"
            "Class of insurance:\n"
            "AUTOMOBILE\n"
            "ANNUAL PREMIUM\n"
            "7$ (excluding tax)\n"
            "YOUR NEW 12-MONTH POLICY\n"
            "MONTANT\n"
            "7$ (excluding tax)\n"
            "TAX\n"
            "0.63$\n"
        )

        self.assertEqual(expected, actual)



# pylint: disable=too-many-locals

import datetime
from unittest.mock import patch

from risc_generator import FPQ1Contract, Protections
from tests.domain.contract.contract_base import ContractTestBase

INCLUDE = "include"
EXCLUDE = "exclude"


class ContractImportantInstructionsTest(ContractTestBase):
    @patch("risc_generator.domain.contract.Faker")
    def test_givenAFRContractNoEndorsements_whenGenerateImportantInstructionText_thenReturnProperFRText(
        self, faker_mock
    ):
        # a_date
        a_year = 2022
        a_month = 1
        a_day = 2
        faker_mock().date_between.return_value = datetime.date(year=a_year, month=a_month, day=a_day)

        a_protections = Protections(protections=self.a_dict_of_protections.copy())

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=self.a_premium_mock,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fpq1_contract.generate_important_instructions_text()

        a_date = "Le 2 janvier 2022\n"
        a_general_phone_number_1_800 = "1 800 123-4567"
        a_new_page = "<###NEW_PAGE###>"

        expected = (
            "INSTRUCTIONS IMPORTANTES\n\n"
            f"{a_date}"
            "Vous trouverez ci-dessous des instructions concernant des éléments très importants de votre police "
            "d'assurance et nous vous recommandons de les lire attentivement et d'y donner suite s'il y a lieu.\n\n"
            "Programme FIDÉLITÉ\n\n"
            "Parce que votre fidélité nous tient à coeur, nous sommes heureux de vous offrir des privilèges qui "
            "s'accumulent en même temps que vos années de fidélité, grâce à notre programme FIDÉLITÉ. "
            "Voici le détail de vos privilèges, que vous trouverez également dans votre Espace client.\n"
            "Votre fidélité reconnue: Nouveau client\n"
            "Vos privilèges:\n"
            "\t Offres exclusives chez nos partenaires\n"
            "\t Assistance juridique\n"
            "\t Service d'aide psychologique\n"
            "Cette liste représente l'ensemble des privilèges associés à votre fidélité reconnue. "
            "Veuillez noter que les nouveaux privilèges seront disponibles au "
            "moment du renouvellement de chacune des polices d'assurance détenues par un "
            "membre de votre maisonnée. Référez-vous aux Conditions "
            "particulières de vos divers contrats d'assurance détenus chez "
            "nous pour connaître le détail des protections applicables.\n"
            f"{a_new_page}\n"
            "\nActions à entreprendre\n\n"
            "Assurance auto\n"
            "\t Détachez et conservez avec vous vos certificats d'assurance.\n"
            "\t Détachez et conservez avec votre permis de conduire le certificat d'assurance pour location de "
            "voiture. Vous éviterez ainsi de débourser les frais d'assurance additionnels auprès de l'agence de "
            "location.\n"
            "\t Veuillez prendre connaissance de la section Conditions particulières de votre police d'assurance "
            "automobile du Québec, afin de vous assurer que vos couvertures répondent bien à vos besoins.\n"
            f"\t N'hésitez pas à joindre le service à la clientèle au {a_general_phone_number_1_800} "
            f"pour toute question relative à votre contrat.\n"
            f"{a_new_page}\n"
        )

        self.assertEqual(expected, actual)

    @patch("risc_generator.domain.contract.Faker")
    def test_givenAFRContractFAQ41_whenGenerateImportantInstructionText_thenReturnProperFRText(self, faker_mock):
        # a_date
        a_year = 2022
        a_month = 1
        a_day = 2
        faker_mock().date_between.return_value = datetime.date(year=a_year, month=a_month, day=a_day)

        dict_protections = self.a_dict_of_protections.copy()
        dict_protections.update({"FAQ41Coverage": INCLUDE})

        protections = Protections(protections=dict_protections)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=protections,
            premium=self.a_premium_mock,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fpq1_contract.generate_important_instructions_text()

        a_date = "Le 2 janvier 2022\n"
        a_general_phone_number_1_800 = "1 800 123-4567"
        a_new_page = "<###NEW_PAGE###>"

        faq41_endorsement_text = "\t Congé de franchise en cas de perte totale ou de délit de fuite (auto)\n"

        expected = (
            "INSTRUCTIONS IMPORTANTES\n\n"
            f"{a_date}"
            "Vous trouverez ci-dessous des instructions concernant des éléments très importants de votre police "
            "d'assurance et nous vous recommandons de les lire attentivement et d'y donner suite s'il y a lieu.\n\n"
            "Programme FIDÉLITÉ\n\n"
            "Parce que votre fidélité nous tient à coeur, nous sommes heureux de vous offrir des privilèges qui "
            "s'accumulent en même temps que vos années de fidélité, grâce à notre programme FIDÉLITÉ. "
            "Voici le détail de vos privilèges, que vous trouverez également dans votre Espace client.\n"
            "Votre fidélité reconnue: Nouveau client\n"
            "Vos privilèges:\n"
            "\t Offres exclusives chez nos partenaires\n"
            "\t Assistance juridique\n"
            "\t Service d'aide psychologique\n"
            f"{faq41_endorsement_text}"
            "Cette liste représente l'ensemble des privilèges associés à votre fidélité reconnue. "
            "Veuillez noter que les nouveaux privilèges seront disponibles au "
            "moment du renouvellement de chacune des polices d'assurance détenues par un "
            "membre de votre maisonnée. Référez-vous aux Conditions "
            "particulières de vos divers contrats d'assurance détenus chez "
            "nous pour connaître le détail des protections applicables.\n"
            f"{a_new_page}\n"
            "\nActions à entreprendre\n\n"
            "Assurance auto\n"
            "\t Détachez et conservez avec vous vos certificats d'assurance.\n"
            "\t Détachez et conservez avec votre permis de conduire le certificat d'assurance pour location de "
            "voiture. Vous éviterez ainsi de débourser les frais d'assurance additionnels auprès de l'agence de "
            "location.\n"
            "\t Veuillez prendre connaissance de la section Conditions particulières de votre police d'assurance "
            "automobile du Québec, afin de vous assurer que vos couvertures répondent bien à vos besoins.\n"
            f"\t N'hésitez pas à joindre le service à la clientèle au {a_general_phone_number_1_800} "
            f"pour toute question relative à votre contrat.\n"
            f"{a_new_page}\n"
        )

        self.assertEqual(expected, actual)

    @patch("risc_generator.domain.contract.Faker")
    def test_givenAFRContractFAQ33_whenGenerateImportantInstructionText_thenReturnProperFRText(self, faker_mock):
        # a_date
        a_year = 2022
        a_month = 1
        a_day = 2
        faker_mock().date_between.return_value = datetime.date(year=a_year, month=a_month, day=a_day)

        dict_protections = self.a_dict_of_protections.copy()
        dict_protections.update({"FAQ33Coverage": INCLUDE})

        protections = Protections(protections=dict_protections)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=protections,
            premium=self.a_premium_mock,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fpq1_contract.generate_important_instructions_text()

        a_date = "Le 2 janvier 2022\n"
        a_general_phone_number_1_800 = "1 800 123-4567"
        a_new_page = "<###NEW_PAGE###>"

        a_roadside_program_name = "ABC"
        a_roadside_phone_number = "1 877 012-3456"
        a_policy_number = "1234567-001"
        faq33_endorsement_text = (
            f"POLICE AUTOMOBILE N {a_policy_number}\n"
            f"Vous avez opté pour {a_roadside_program_name} assistance routière, qui vous procure des services "
            f"de dépannage partout au Canada et aux États-Unis, 24 heures sur 24, 365 jours par année.\n"
            f"Ainsi, trois semaines après la date de prise d'effet de l'avenant F.A.Q. N 33 Assurance des frais "
            f"de dépannage, vous recevrez une trousse pour chaque véhicule admissible, contenant entre autres "
            f"votre carte d'adhérent ainsi que votre guide de l'usager.\n"
            f"Dès que l'avenant F.A.Q. N 33 entrera en vigueur, vous pourrez bénéficier de ce service même si "
            f"vous n'avez pas reçu votre trousse {a_roadside_program_name} assistance routière. En cas de besoin, "
            f"il vous suffira de composer le {a_roadside_phone_number} (24 heures sur 24, 7 jours sur 7) "
            f"et de mentionner le numéro de police du véhicule couvert au préposé.\n"
        )

        expected = (
            "INSTRUCTIONS IMPORTANTES\n\n"
            f"{a_date}"
            "Vous trouverez ci-dessous des instructions concernant des éléments très importants de votre police "
            "d'assurance et nous vous recommandons de les lire attentivement et d'y donner suite s'il y a lieu.\n\n"
            "Programme FIDÉLITÉ\n\n"
            "Parce que votre fidélité nous tient à coeur, nous sommes heureux de vous offrir des privilèges qui "
            "s'accumulent en même temps que vos années de fidélité, grâce à notre programme FIDÉLITÉ. "
            "Voici le détail de vos privilèges, que vous trouverez également dans votre Espace client.\n"
            "Votre fidélité reconnue: Nouveau client\n"
            "Vos privilèges:\n"
            "\t Offres exclusives chez nos partenaires\n"
            "\t Assistance juridique\n"
            "\t Service d'aide psychologique\n"
            "Cette liste représente l'ensemble des privilèges associés à votre fidélité reconnue. "
            "Veuillez noter que les nouveaux privilèges seront disponibles au "
            "moment du renouvellement de chacune des polices d'assurance détenues par un "
            "membre de votre maisonnée. Référez-vous aux Conditions "
            "particulières de vos divers contrats d'assurance détenus chez "
            "nous pour connaître le détail des protections applicables.\n"
            "Avis à l'assuré\n"
            f"{faq33_endorsement_text}"
            f"{a_new_page}\n"
            "\nActions à entreprendre\n\n"
            "Assurance auto\n"
            "\t Détachez et conservez avec vous vos certificats d'assurance.\n"
            "\t Détachez et conservez avec votre permis de conduire le certificat d'assurance pour location de "
            "voiture. Vous éviterez ainsi de débourser les frais d'assurance additionnels auprès de l'agence de "
            "location.\n"
            "\t Veuillez prendre connaissance de la section Conditions particulières de votre police d'assurance "
            "automobile du Québec, afin de vous assurer que vos couvertures répondent bien à vos besoins.\n"
            f"\t N'hésitez pas à joindre le service à la clientèle au {a_general_phone_number_1_800} "
            f"pour toute question relative à votre contrat.\n"
            f"{a_new_page}\n"
        )

        self.assertEqual(expected, actual)

    @patch("risc_generator.domain.contract.Faker")
    def test_givenAFRContractFAQ5A_whenGenerateImportantInstructionText_thenReturnProperFRText(self, faker_mock):
        # a_date
        a_year = 2022
        a_month = 1
        a_day = 2
        faker_mock().date_between.return_value = datetime.date(year=a_year, month=a_month, day=a_day)

        dict_protections = self.a_dict_of_protections.copy()
        dict_protections.update({"FAQ5ACoverage": INCLUDE})

        protections = Protections(protections=dict_protections)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=protections,
            premium=self.a_premium_mock,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fpq1_contract.generate_important_instructions_text()

        a_date = "Le 2 janvier 2022\n"
        a_general_phone_number_1_800 = "1 800 123-4567"
        a_new_page = "<###NEW_PAGE###>"

        a_location_car = "locateur"
        a_policy_number = "1234567-001"
        a_vehicle = "AMaker AModel 2020"
        faq5a_endorsement_text = (
            f"POLICE AUTOMOBILE N {a_policy_number} - Véhicule 1 : {a_vehicle}\n"
            f"Nous joignons une copie de votre police à remettre à votre {a_location_car}, s'il la demande. Elle se "
            f"trouve à la toute fin du présent envoi.\n"
        )

        expected = (
            "INSTRUCTIONS IMPORTANTES\n\n"
            f"{a_date}"
            "Vous trouverez ci-dessous des instructions concernant des éléments très importants de votre police "
            "d'assurance et nous vous recommandons de les lire attentivement et d'y donner suite s'il y a lieu.\n\n"
            "Programme FIDÉLITÉ\n\n"
            "Parce que votre fidélité nous tient à coeur, nous sommes heureux de vous offrir des privilèges qui "
            "s'accumulent en même temps que vos années de fidélité, grâce à notre programme FIDÉLITÉ. "
            "Voici le détail de vos privilèges, que vous trouverez également dans votre Espace client.\n"
            "Votre fidélité reconnue: Nouveau client\n"
            "Vos privilèges:\n"
            "\t Offres exclusives chez nos partenaires\n"
            "\t Assistance juridique\n"
            "\t Service d'aide psychologique\n"
            "Cette liste représente l'ensemble des privilèges associés à votre fidélité reconnue. "
            "Veuillez noter que les nouveaux privilèges seront disponibles au "
            "moment du renouvellement de chacune des polices d'assurance détenues par un "
            "membre de votre maisonnée. Référez-vous aux Conditions "
            "particulières de vos divers contrats d'assurance détenus chez "
            "nous pour connaître le détail des protections applicables.\n"
            "Avis à l'assuré\n"
            f"{faq5a_endorsement_text}"
            f"{a_new_page}\n"
            "\nActions à entreprendre\n\n"
            "Assurance auto\n"
            "\t Détachez et conservez avec vous vos certificats d'assurance.\n"
            "\t Détachez et conservez avec votre permis de conduire le certificat d'assurance pour location de "
            "voiture. Vous éviterez ainsi de débourser les frais d'assurance additionnels auprès de l'agence de "
            "location.\n"
            "\t Veuillez prendre connaissance de la section Conditions particulières de votre police d'assurance "
            "automobile du Québec, afin de vous assurer que vos couvertures répondent bien à vos besoins.\n"
            f"\t N'hésitez pas à joindre le service à la clientèle au {a_general_phone_number_1_800} "
            f"pour toute question relative à votre contrat.\n"
            f"{a_new_page}\n"
        )

        self.assertEqual(expected, actual)

    @patch("risc_generator.domain.contract.Faker")
    def test_givenAFRContractFAQ23A_whenGenerateImportantInstructionText_thenReturnProperFRText(self, faker_mock):
        # a_date
        a_year = 2022
        a_month = 1
        a_day = 2
        faker_mock().date_between.return_value = datetime.date(year=a_year, month=a_month, day=a_day)

        dict_protections = self.a_dict_of_protections.copy()
        dict_protections.update({"FAQ23ACoverage": INCLUDE})

        protections = Protections(protections=dict_protections)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=protections,
            premium=self.a_premium_mock,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fpq1_contract.generate_important_instructions_text()

        a_date = "Le 2 janvier 2022\n"
        a_general_phone_number_1_800 = "1 800 123-4567"
        a_new_page = "<###NEW_PAGE###>"

        a_financed_car = "créancier"
        a_policy_number = "1234567-001"
        a_vehicle = "AMaker AModel 2020"
        faq23a_endorsement_text = (
            f"POLICE AUTOMOBILE N {a_policy_number} - Véhicule 1 : {a_vehicle}\n"
            f"Nous joignons une copie de votre police à remettre à votre {a_financed_car}, s'il la demande. Elle se "
            f"trouve à la toute fin du présent envoi.\n"
        )

        expected = (
            "INSTRUCTIONS IMPORTANTES\n\n"
            f"{a_date}"
            "Vous trouverez ci-dessous des instructions concernant des éléments très importants de votre police "
            "d'assurance et nous vous recommandons de les lire attentivement et d'y donner suite s'il y a lieu.\n\n"
            "Programme FIDÉLITÉ\n\n"
            "Parce que votre fidélité nous tient à coeur, nous sommes heureux de vous offrir des privilèges qui "
            "s'accumulent en même temps que vos années de fidélité, grâce à notre programme FIDÉLITÉ. "
            "Voici le détail de vos privilèges, que vous trouverez également dans votre Espace client.\n"
            "Votre fidélité reconnue: Nouveau client\n"
            "Vos privilèges:\n"
            "\t Offres exclusives chez nos partenaires\n"
            "\t Assistance juridique\n"
            "\t Service d'aide psychologique\n"
            "Cette liste représente l'ensemble des privilèges associés à votre fidélité reconnue. "
            "Veuillez noter que les nouveaux privilèges seront disponibles au "
            "moment du renouvellement de chacune des polices d'assurance détenues par un "
            "membre de votre maisonnée. Référez-vous aux Conditions "
            "particulières de vos divers contrats d'assurance détenus chez "
            "nous pour connaître le détail des protections applicables.\n"
            "Avis à l'assuré\n"
            f"{faq23a_endorsement_text}"
            f"{a_new_page}\n"
            "\nActions à entreprendre\n\n"
            "Assurance auto\n"
            "\t Détachez et conservez avec vous vos certificats d'assurance.\n"
            "\t Détachez et conservez avec votre permis de conduire le certificat d'assurance pour location de "
            "voiture. Vous éviterez ainsi de débourser les frais d'assurance additionnels auprès de l'agence de "
            "location.\n"
            "\t Veuillez prendre connaissance de la section Conditions particulières de votre police d'assurance "
            "automobile du Québec, afin de vous assurer que vos couvertures répondent bien à vos besoins.\n"
            f"\t N'hésitez pas à joindre le service à la clientèle au {a_general_phone_number_1_800} "
            f"pour toute question relative à votre contrat.\n"
            f"{a_new_page}\n"
        )

        self.assertEqual(expected, actual)

    @patch("risc_generator.domain.contract.Faker")
    def test_givenAFRContractFAQ33And23A_whenGenerateImportantInstructionText_thenReturnProperFRText(self, faker_mock):
        # a_date
        a_year = 2022
        a_month = 1
        a_day = 2
        faker_mock().date_between.return_value = datetime.date(year=a_year, month=a_month, day=a_day)

        dict_protections = self.a_dict_of_protections.copy()
        dict_protections.update({"FAQ33Coverage": INCLUDE})
        dict_protections.update({"FAQ23ACoverage": INCLUDE})

        protections = Protections(protections=dict_protections)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=protections,
            premium=self.a_premium_mock,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fpq1_contract.generate_important_instructions_text()

        a_date = "Le 2 janvier 2022\n"
        a_general_phone_number_1_800 = "1 800 123-4567"
        a_new_page = "<###NEW_PAGE###>"

        a_financed_car = "créancier"
        a_policy_number = "1234567-001"
        a_vehicle = "AMaker AModel 2020"
        a_roadside_program_name = "ABC"
        a_roadside_phone_number = "1 877 012-3456"
        endorsements_text = (
            f"POLICE AUTOMOBILE N {a_policy_number}\n"
            f"Vous avez opté pour {a_roadside_program_name} assistance routière, qui vous procure des services "
            f"de dépannage partout au Canada et aux États-Unis, 24 heures sur 24, 365 jours par année.\n"
            f"Ainsi, trois semaines après la date de prise d'effet de l'avenant F.A.Q. N 33 Assurance des frais "
            f"de dépannage, vous recevrez une trousse pour chaque véhicule admissible, contenant entre autres "
            f"votre carte d'adhérent ainsi que votre guide de l'usager.\n"
            f"Dès que l'avenant F.A.Q. N 33 entrera en vigueur, vous pourrez bénéficier de ce service même si "
            f"vous n'avez pas reçu votre trousse {a_roadside_program_name} assistance routière. En cas de besoin, "
            f"il vous suffira de composer le {a_roadside_phone_number} (24 heures sur 24, 7 jours sur 7) "
            f"et de mentionner le numéro de police du véhicule couvert au préposé.\n"
            f"POLICE AUTOMOBILE N {a_policy_number} - Véhicule 1 : {a_vehicle}\n"
            f"Nous joignons une copie de votre police à remettre à votre {a_financed_car}, s'il la demande. Elle se "
            f"trouve à la toute fin du présent envoi.\n"
        )

        expected = (
            "INSTRUCTIONS IMPORTANTES\n\n"
            f"{a_date}"
            "Vous trouverez ci-dessous des instructions concernant des éléments très importants de votre police "
            "d'assurance et nous vous recommandons de les lire attentivement et d'y donner suite s'il y a lieu.\n\n"
            "Programme FIDÉLITÉ\n\n"
            "Parce que votre fidélité nous tient à coeur, nous sommes heureux de vous offrir des privilèges qui "
            "s'accumulent en même temps que vos années de fidélité, grâce à notre programme FIDÉLITÉ. "
            "Voici le détail de vos privilèges, que vous trouverez également dans votre Espace client.\n"
            "Votre fidélité reconnue: Nouveau client\n"
            "Vos privilèges:\n"
            "\t Offres exclusives chez nos partenaires\n"
            "\t Assistance juridique\n"
            "\t Service d'aide psychologique\n"
            "Cette liste représente l'ensemble des privilèges associés à votre fidélité reconnue. "
            "Veuillez noter que les nouveaux privilèges seront disponibles au "
            "moment du renouvellement de chacune des polices d'assurance détenues par un "
            "membre de votre maisonnée. Référez-vous aux Conditions "
            "particulières de vos divers contrats d'assurance détenus chez "
            "nous pour connaître le détail des protections applicables.\n"
            "Avis à l'assuré\n"
            f"{endorsements_text}"
            f"{a_new_page}\n"
            "\nActions à entreprendre\n\n"
            "Assurance auto\n"
            "\t Détachez et conservez avec vous vos certificats d'assurance.\n"
            "\t Détachez et conservez avec votre permis de conduire le certificat d'assurance pour location de "
            "voiture. Vous éviterez ainsi de débourser les frais d'assurance additionnels auprès de l'agence de "
            "location.\n"
            "\t Veuillez prendre connaissance de la section Conditions particulières de votre police d'assurance "
            "automobile du Québec, afin de vous assurer que vos couvertures répondent bien à vos besoins.\n"
            f"\t N'hésitez pas à joindre le service à la clientèle au {a_general_phone_number_1_800} "
            f"pour toute question relative à votre contrat.\n"
            f"{a_new_page}\n"
        )

        self.assertEqual(expected, actual)

    @patch("risc_generator.domain.contract.Faker")
    def test_givenAENContractNoEndorsements_whenGenerateImportantInstructionText_thenReturnProperENText(
        self, faker_mock
    ):
        # a_date
        a_year = 2022
        a_month = 1
        a_day = 2
        faker_mock().date_between.return_value = datetime.date(year=a_year, month=a_month, day=a_day)

        a_protections = Protections(protections=self.a_dict_of_protections.copy())

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=self.a_premium_mock,
            financing=self.a_creditor,
            locale=self.a_en_locale,
        )

        actual = fpq1_contract.generate_important_instructions_text()

        a_date = "January 2, 2022\n"
        a_general_phone_number_1_800 = "1 800 123-4567"
        a_new_page = "<###NEW_PAGE###>"

        expected = (
            "IMPORTANT INSTRUCTIONS\n\n"
            f"{a_date}"
            "Please find hereafter instructions on very important parts of your insurance policy; we suggest that "
            "you read them carefully and take any required action, if necessary.\n\n"
            "FIDELITY Program\n\n"
            "Because we appreciate your loyalty, we're pleased to offer you privileges that accumulate with your "
            "years of loyalty, thanks to our FIDELITY Program. Here are the details regarding your privileges, "
            "which you can also find in your Client Centre.\n"
            "Your recognized loyalty: New client\n"
            "Your privileges:\n"
            "\t Exclusive partner offers\n"
            "\t Legal assistance\n"
            "\t Psychological assistance service\n"
            "This list contains all the privileges associated with your recognized loyalty. Please note that the new "
            "privileges will be available at the renewal of each insurance policy held by a member of your household. "
            "Please see the Declarations or Coverage Summary Page(s) of your various insurance contracts held with "
            "us to learn more about applicable coverage details.\n"
            f"{a_new_page}\n"
            "\nActions Required\n\n"
            "Automobile insurance\n"
            "\t Detach and keep your insurance certificates with you.\n"
            "\t Detach and keep the automobile rental certificate with your driver's licence. This way, you will "
            "avoid paying additional insurance fees to the car rental agency.\n"
            "\t Please read the Coverage Summary section of your Quebec automobile insurance contract to ensure "
            "that your coverages are well-suited to your needs.\n"
            f"\t Don't hesitate to contact customer service at {a_general_phone_number_1_800} for any questions "
            f"regarding your contract.\n"
            f"{a_new_page}\n"
        )

        self.assertEqual(expected, actual)

    @patch("risc_generator.domain.contract.Faker")
    def test_givenAENContractFAQ41_whenGenerateImportantInstructionText_thenReturnProperENText(self, faker_mock):
        # a_date
        a_year = 2022
        a_month = 1
        a_day = 2
        faker_mock().date_between.return_value = datetime.date(year=a_year, month=a_month, day=a_day)

        dict_protections = self.a_dict_of_protections.copy()
        dict_protections.update({"FAQ41Coverage": INCLUDE})

        protections = Protections(protections=dict_protections)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=protections,
            premium=self.a_premium_mock,
            financing=self.a_creditor,
            locale=self.a_en_locale,
        )

        actual = fpq1_contract.generate_important_instructions_text()

        a_date = "January 2, 2022\n"
        a_general_phone_number_1_800 = "1 800 123-4567"
        a_new_page = "<###NEW_PAGE###>"

        faq41_endorsement_text = "\t No deductible to pay in the case of a total loss or hit-and-run (auto)\n"

        expected = (
            "IMPORTANT INSTRUCTIONS\n\n"
            f"{a_date}"
            "Please find hereafter instructions on very important parts of your insurance policy; we suggest that "
            "you read them carefully and take any required action, if necessary.\n\n"
            "FIDELITY Program\n\n"
            "Because we appreciate your loyalty, we're pleased to offer you privileges that accumulate with your "
            "years of loyalty, thanks to our FIDELITY Program. Here are the details regarding your privileges, "
            "which you can also find in your Client Centre.\n"
            "Your recognized loyalty: New client\n"
            "Your privileges:\n"
            "\t Exclusive partner offers\n"
            "\t Legal assistance\n"
            "\t Psychological assistance service\n"
            f"{faq41_endorsement_text}"
            "This list contains all the privileges associated with your recognized loyalty. Please note that the new "
            "privileges will be available at the renewal of each insurance policy held by a member of your household. "
            "Please see the Declarations or Coverage Summary Page(s) of your various insurance contracts held with "
            "us to learn more about applicable coverage details.\n"
            f"{a_new_page}\n"
            "\nActions Required\n\n"
            "Automobile insurance\n"
            "\t Detach and keep your insurance certificates with you.\n"
            "\t Detach and keep the automobile rental certificate with your driver's licence. This way, you will "
            "avoid paying additional insurance fees to the car rental agency.\n"
            "\t Please read the Coverage Summary section of your Quebec automobile insurance contract to ensure "
            "that your coverages are well-suited to your needs.\n"
            f"\t Don't hesitate to contact customer service at {a_general_phone_number_1_800} for any questions "
            f"regarding your contract.\n"
            f"{a_new_page}\n"
        )

        self.assertEqual(expected, actual)

    @patch("risc_generator.domain.contract.Faker")
    def test_givenAENContractFAQ33_whenGenerateImportantInstructionText_thenReturnProperENText(self, faker_mock):
        # a_date
        a_year = 2022
        a_month = 1
        a_day = 2
        faker_mock().date_between.return_value = datetime.date(year=a_year, month=a_month, day=a_day)

        dict_protections = self.a_dict_of_protections.copy()
        dict_protections.update({"FAQ33Coverage": INCLUDE})

        protections = Protections(protections=dict_protections)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=protections,
            premium=self.a_premium_mock,
            financing=self.a_creditor,
            locale=self.a_en_locale,
        )

        actual = fpq1_contract.generate_important_instructions_text()

        a_date = "January 2, 2022\n"
        a_general_phone_number_1_800 = "1 800 123-4567"
        a_new_page = "<###NEW_PAGE###>"

        a_roadside_program_name = "ABC"
        a_roadside_phone_number = "1 877 012-3456"
        a_policy_number = "1234567-001"
        faq33_endorsement_text = (
            f"AUTOMOBILE POLICY No {a_policy_number}\n"
            f"You have chosen {a_roadside_program_name} Roadside Assistance, which provides you with 24-hour "
            f"roadside assistance throughout Canada and the United States, 365 days a year.\n"
            f"Therefore, three weeks after the effective date of the Q.E.F. endorsement N 33 Roadside Assistance "
            f"Coverage, you will receive a kit for each eligible vehicle, including your membership card and your "
            f"user's guide.\n"
            f"As soon as the endorsement Q.E.F. N 33 comes into effect, you will be able to benefit from "
            f"this service even if you have not received your {a_roadside_program_name} roadside assistance kit."
            f"In case of need, simply call {a_roadside_phone_number} (24 hours a day, 7 days a week) "
            f"and mention the policy number of the covered vehicle to the agent.\n"
        )

        expected = (
            "IMPORTANT INSTRUCTIONS\n\n"
            f"{a_date}"
            "Please find hereafter instructions on very important parts of your insurance policy; we suggest that "
            "you read them carefully and take any required action, if necessary.\n\n"
            "FIDELITY Program\n\n"
            "Because we appreciate your loyalty, we're pleased to offer you privileges that accumulate with your "
            "years of loyalty, thanks to our FIDELITY Program. Here are the details regarding your privileges, "
            "which you can also find in your Client Centre.\n"
            "Your recognized loyalty: New client\n"
            "Your privileges:\n"
            "\t Exclusive partner offers\n"
            "\t Legal assistance\n"
            "\t Psychological assistance service\n"
            "This list contains all the privileges associated with your recognized loyalty. Please note that the new "
            "privileges will be available at the renewal of each insurance policy held by a member of your household. "
            "Please see the Declarations or Coverage Summary Page(s) of your various insurance contracts held with "
            "us to learn more about applicable coverage details.\n"
            "Notice to the insured\n"
            f"{faq33_endorsement_text}"
            f"{a_new_page}\n"
            "\nActions Required\n\n"
            "Automobile insurance\n"
            "\t Detach and keep your insurance certificates with you.\n"
            "\t Detach and keep the automobile rental certificate with your driver's licence. This way, you will "
            "avoid paying additional insurance fees to the car rental agency.\n"
            "\t Please read the Coverage Summary section of your Quebec automobile insurance contract to ensure "
            "that your coverages are well-suited to your needs.\n"
            f"\t Don't hesitate to contact customer service at {a_general_phone_number_1_800} for any questions "
            f"regarding your contract.\n"
            f"{a_new_page}\n"
        )

        self.assertEqual(expected, actual)

    @patch("risc_generator.domain.contract.Faker")
    def test_givenAENContractFAQ5A_whenGenerateImportantInstructionText_thenReturnProperENText(self, faker_mock):
        # a_date
        a_year = 2022
        a_month = 1
        a_day = 2
        faker_mock().date_between.return_value = datetime.date(year=a_year, month=a_month, day=a_day)

        dict_protections = self.a_dict_of_protections.copy()
        dict_protections.update({"FAQ5ACoverage": INCLUDE})

        protections = Protections(protections=dict_protections)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=protections,
            premium=self.a_premium_mock,
            financing=self.a_creditor,
            locale=self.a_en_locale,
        )

        actual = fpq1_contract.generate_important_instructions_text()

        a_date = "January 2, 2022\n"
        a_general_phone_number_1_800 = "1 800 123-4567"
        a_new_page = "<###NEW_PAGE###>"

        a_location_car = "locator"
        a_policy_number = "1234567-001"
        a_vehicle = "AMaker AModel 2020"
        faq5a_endorsement_text = (
            f"AUTOMOBILE POLICY No {a_policy_number} - Vehicle 1 : {a_vehicle}\n"
            f"We are enclosing a copy of your policy to give to your {a_location_car}, "
            f"if requested. It is included at the very end of this mailing.\n"
        )

        expected = (
            "IMPORTANT INSTRUCTIONS\n\n"
            f"{a_date}"
            "Please find hereafter instructions on very important parts of your insurance policy; we suggest that "
            "you read them carefully and take any required action, if necessary.\n\n"
            "FIDELITY Program\n\n"
            "Because we appreciate your loyalty, we're pleased to offer you privileges that accumulate with your "
            "years of loyalty, thanks to our FIDELITY Program. Here are the details regarding your privileges, "
            "which you can also find in your Client Centre.\n"
            "Your recognized loyalty: New client\n"
            "Your privileges:\n"
            "\t Exclusive partner offers\n"
            "\t Legal assistance\n"
            "\t Psychological assistance service\n"
            "This list contains all the privileges associated with your recognized loyalty. Please note that the new "
            "privileges will be available at the renewal of each insurance policy held by a member of your household. "
            "Please see the Declarations or Coverage Summary Page(s) of your various insurance contracts held with "
            "us to learn more about applicable coverage details.\n"
            "Notice to the insured\n"
            f"{faq5a_endorsement_text}"
            f"{a_new_page}\n"
            "\nActions Required\n\n"
            "Automobile insurance\n"
            "\t Detach and keep your insurance certificates with you.\n"
            "\t Detach and keep the automobile rental certificate with your driver's licence. This way, you will "
            "avoid paying additional insurance fees to the car rental agency.\n"
            "\t Please read the Coverage Summary section of your Quebec automobile insurance contract to ensure "
            "that your coverages are well-suited to your needs.\n"
            f"\t Don't hesitate to contact customer service at {a_general_phone_number_1_800} for any questions "
            f"regarding your contract.\n"
            f"{a_new_page}\n"
        )

        self.assertEqual(expected, actual)

    @patch("risc_generator.domain.contract.Faker")
    def test_givenAENContractFAQ23A_whenGenerateImportantInstructionText_thenReturnProperENText(self, faker_mock):
        # a_date
        a_year = 2022
        a_month = 1
        a_day = 2
        faker_mock().date_between.return_value = datetime.date(year=a_year, month=a_month, day=a_day)

        dict_protections = self.a_dict_of_protections.copy()
        dict_protections.update({"FAQ23ACoverage": INCLUDE})

        protections = Protections(protections=dict_protections)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=protections,
            premium=self.a_premium_mock,
            financing=self.a_creditor,
            locale=self.a_en_locale,
        )

        actual = fpq1_contract.generate_important_instructions_text()

        a_date = "January 2, 2022\n"
        a_general_phone_number_1_800 = "1 800 123-4567"
        a_new_page = "<###NEW_PAGE###>"

        a_financed_car = "creditor"
        a_policy_number = "1234567-001"
        a_vehicle = "AMaker AModel 2020"
        faq23a_endorsement_text = (
            f"AUTOMOBILE POLICY No {a_policy_number} - Vehicle 1 : {a_vehicle}\n"
            f"We are enclosing a copy of your policy to give to your {a_financed_car}, "
            f"if requested. It is included at the very end of this mailing.\n"
        )

        expected = (
            "IMPORTANT INSTRUCTIONS\n\n"
            f"{a_date}"
            "Please find hereafter instructions on very important parts of your insurance policy; we suggest that "
            "you read them carefully and take any required action, if necessary.\n\n"
            "FIDELITY Program\n\n"
            "Because we appreciate your loyalty, we're pleased to offer you privileges that accumulate with your "
            "years of loyalty, thanks to our FIDELITY Program. Here are the details regarding your privileges, "
            "which you can also find in your Client Centre.\n"
            "Your recognized loyalty: New client\n"
            "Your privileges:\n"
            "\t Exclusive partner offers\n"
            "\t Legal assistance\n"
            "\t Psychological assistance service\n"
            "This list contains all the privileges associated with your recognized loyalty. Please note that the new "
            "privileges will be available at the renewal of each insurance policy held by a member of your household. "
            "Please see the Declarations or Coverage Summary Page(s) of your various insurance contracts held with "
            "us to learn more about applicable coverage details.\n"
            "Notice to the insured\n"
            f"{faq23a_endorsement_text}"
            f"{a_new_page}\n"
            "\nActions Required\n\n"
            "Automobile insurance\n"
            "\t Detach and keep your insurance certificates with you.\n"
            "\t Detach and keep the automobile rental certificate with your driver's licence. This way, you will "
            "avoid paying additional insurance fees to the car rental agency.\n"
            "\t Please read the Coverage Summary section of your Quebec automobile insurance contract to ensure "
            "that your coverages are well-suited to your needs.\n"
            f"\t Don't hesitate to contact customer service at {a_general_phone_number_1_800} for any questions "
            f"regarding your contract.\n"
            f"{a_new_page}\n"
        )

        self.assertEqual(expected, actual)



from random import choice

from faker import Faker

bank_names_fr = [
    "Banque CIBC",
    "BMO Banque de Montréal",
    "Banque Desjardins",
    "Banque HSBC Canada",
    "Banque Laurentienne du Canada",
    "Banque Nationale du Canada",
    "Banque Royale du Canada",
    "Banque Scotia",
    "Banque TD Canada Trust",
]

bank_names_en = [
    "CIBC Bank",
    "BMO Montreal Bank",
    "Desjardins Bank",
    "HSBC Canada Bank",
    "Laurentian Bank of Canada",
    "National Bank of Canada",
    "Royal Bank of Canada",
    "Scotia Bank",
    "TD Canada Trust Bank",
]


class FinancingFaker:
    def __init__(self, locale: str) -> None:
        """
        A faker to fake a financing institution.

                locale (str): The locale language setting to use for simulation. Can either be `'fr_CA'` or `'en_CA'`.
        """
        self.locale = locale

        self.address_faker = Faker(locale=self.locale)

        if self.locale == "fr_CA":
            self.bank_name = bank_names_fr
        elif self.locale == "en_CA":
            self.bank_name = bank_names_en
        else:
            raise ValueError(f"The locale {locale} is not supporter. It can either be 'fr_CA' or 'en_CA'.")

    def financing(self) -> str:
        """
        Method to fake a financing details information. Namely, is name and address.

        Return:
            A string of the bank name and address in capitalize characters.
        """
        bank = choice(self.bank_name)
        address = self.address_faker.address()

        return f"{bank} {address}"



from ..endorsements.endorsement_header import generate_endorsement_page_header_text
from ...config import NEW_PAGE_TAG


@staticmethod
def generate_faq8_endorsement_text() -> str:
    endorsement_header_text = generate_endorsement_page_header_text()
    text = (
        f"{endorsement_header_text}"
        "Formulaire d'avenant du Québec\n"
        "F.A.Q. N 8\n"
        "Franchise pour les dommages matériels\n"
        "(Chapitre A)\n"
        "Le titre de l'avenant doit être écrit à la section « Conditions particulières » du contrat "
        "d'assurance. Quant aux informations requises dans l'avenant, elles peuvent être écrites à "
        "cette section ou dans l'avenant même, au choix de l'assureur.\n"
        "Nom de l'assureur : \n"
        "Nom de l'assuré désigné : \n"
        "Avenant à la police d'assurance automobile N :\n"
        "Date de prise d'effet : cet avenant s'applique à partir du à 0 h 01, heure normale "
        "à l'adresse de l'assuré désigné.\n"
        "Véhicule visé : cet avenant s'applique uniquement au véhicule désigné suivant :\n"
        "(numéro de référence écrit à la section « Conditions particulières » du contrat d'assurance)\n"
        "Description de l'avenant\n"
        "Cet avenant modifie le chapitre A du contrat d'assurance en y ajoutant une franchise pour les "
        "dommages matériels causés par le véhicule visé :\n"
        "\t Une franchise d'un montant maximum de  $ par sinistre.\n"
        "\t Une franchise d'un montant maximum de  $ par sinistre, lorsque le véhicule est utilisé pour .\n"
        "Engagement de l'assuré désigné\n"
        "Lorsque l'assureur paie une indemnité pour des dommages matériels, l'assuré désigné "
        "s'engage à rembourser l'assureur jusqu'à concurrence du montant de la franchise.\n"
        "Le montant à rembourser est exigible de l'assuré désigné dès que l'assureur paie l'indemnité.\n"
        "Droits de l'assureur\n"
        "À l'égard de la franchise, l'assureur a le droit :\n"
        "\t d'agir comme il le veut en matière d'enquête, de transaction ou de règlement;\n"
        "\t d'autoriser l'assuré désigné à conclure une transaction ou un règlement avec "
        "toute autre personne qui a subi un dommage matériel et à indemniser cette "
        "personne. Par contre, le total des montants convenus à la suite de la "
        "transaction ou du règlement ne doit pas dépasser le montant de la franchise.\n"
        "Toutes les autres conditions du contrat d'assurance restent les mêmes.\n"
        f"{NEW_PAGE_TAG}\n"
    )

    return text
